def build_payload(
    summary,
    description,
    issue_type,
    project_key,
    affects_version,
    component_name,
    priority,
    epic_field,
):
    fields = {
        "project": {"key": project_key},
        "summary": summary,
        "description": description,
        "issuetype": {"name": issue_type.capitalize()},
        "priority": {"name": priority},
        "versions": [{"name": affects_version}],
        "components": [{"name": component_name}],
    }

    if issue_type.lower() == "epic":
        fields[epic_field] = summary

    return {"fields": fields}



def set_priority(request_fn, issue_key, priority):
    # Put this somewhere else
    priorities = {
        "critical": "Critical",
        "major": "Major",
        "normal": "Normal",
        "minor": "Minor",
    }

    priority = (
        priorities[priority.lower()] if priority.lower() in priorities else "Normal"
    )

    request_fn(
        "PUT",
        f"/rest/api/2/issue/{issue_key}",
        json={"fields": {"priority": {"name": priority}}},
    )



import os
import subprocess
import tempfile

from exceptions.exceptions import AddCommentError, AiError


def cli_add_comment(jira, ai_provider, default_prompt, args):
    if args.text:
        comment = args.text
    else:
        with tempfile.NamedTemporaryFile(mode="w+", suffix=".md", delete=False) as tmp:
            tmp.write("# Enter comment below\n")
            tmp.flush()
            subprocess.call([os.environ.get("EDITOR", "vim"), tmp.name])
            tmp.seek(0)
            comment = tmp.read()

    if not comment.strip():
        print("⚠️ No comment provided. Skipping.")
        return False

    try:
        cleaned = ai_provider.improve_text(default_prompt, comment)
    except AiError as e:
        msg = f"⚠️ AI cleanup failed. Using raw comment. Error: {e}"
        print(msg)
        raise (AiError(msg))

    try:
        jira.add_comment(args.issue_key, cleaned)
        print(f"✅ Comment added to {args.issue_key}")
        return True
    except AddCommentError as e:
        msg = f"❌ Failed to add comment: {e}"
        print(msg)
        raise (AddCommentError(msg))



import os

import requests


class BARTProvider:
    def __init__(self):
        # Default to local endpoint or override with env var
        self.url = os.getenv("AI_URL", "http://localhost:8000/bart")
        self.headers = {"Content-Type": "application/json"}

    def improve_text(self, prompt: str, text: str) -> str:
        full_prompt = f"{prompt}\n\n{text}"
        response = requests.post(
            self.url, headers=self.headers, json={"text": full_prompt}, timeout=30
        )
        if response.status_code == 200:
            return response.json().get("output", "").strip()

        raise Exception(
            f"BART request failed: {response.status_code} - {response.text}"
        )



import re

from core.env_fetcher import EnvFetcher


def search_issues(request_fn, jql):
    params = {
        "jql": jql,
        "fields": (
            "summary,status,assignee,priority,"
            + EnvFetcher.get("JIRA_STORY_POINTS_FIELD")
            + ","
            + EnvFetcher.get("JIRA_SPRINT_FIELD")
            + ","
            + EnvFetcher.get("JIRA_BLOCKED_FIELD")
        ),
        "maxResults": 200,
    }

    issues = request_fn("GET", "/rest/api/2/search", params=params).get("issues", [])

    name_regex = r"name\s*=\s*([^,]+)"
    state_regex = r"state\s*=\s*([A-Za-z]+)"

    for issue in issues:
        sprints = issue.get("fields", {}).get(EnvFetcher.get("JIRA_SPRINT_FIELD"), [])

        if not sprints:
            issue["fields"]["sprint"] = "No active sprint"
            continue

        active_sprint = None
        for sprint_str in sprints:
            name_match = re.search(name_regex, sprint_str)
            sprint_name = name_match.group(1) if name_match else None

            state_match = re.search(state_regex, sprint_str)
            sprint_state = state_match.group(1) if state_match else None

            if sprint_state == "ACTIVE" and sprint_name:
                active_sprint = sprint_name
                break

        issue["fields"]["sprint"] = (
            active_sprint if active_sprint else "No active sprint"
        )

    return issues



from unittest.mock import MagicMock, patch

from core.env_fetcher import EnvFetcher
from exceptions.exceptions import LintAllError

import pytest  # isort: skip
from commands.cli_lint_all import print_status_table  # isort: skip


# Ensure the Args object has the required 'project' and other attributes
class Args:
    project = "TestProject"  # Add the required 'project' attribute
    component = "analytics-hcc-service"
    reporter = None
    assignee = None


class ArgsReporter:
    project = "TestProject"  # Add the required 'project' attribute
    component = "analytics-hcc-service"
    reporter = "test"
    assignee = None


class ArgsAssignee:
    project = "TestProject"  # Add the required 'project' attribute
    component = "analytics-hcc-service"
    reporter = None
    assignee = "test"


def test_print_status_table_with_wrapping(capsys):
    # Prepare the mock data
    failure_statuses = [
        {
            "key": "AAP-test_print_status_table_with_wrapping-1",
            "summary": """This is a test summary that exceeds 120 characters
            to check the wrapping functionality of the print function. It should
            not split in the middle of a word.""",
            "progress": True,
        },
        {
            "key": "AAP-test_print_status_table_with_wrapping-2",
            "summary": "This summary is short.",
            "progress": False,
        },
        {
            "key": "AAP-test_print_status_table_with_wrapping-3",
            "summary": "This summary is short.",
            "progress": None,
        },
    ]

    # Call the function with the mock data
    print_status_table(failure_statuses)

    # Capture the output
    captured = capsys.readouterr()
    # Check if the correct symbols for progress are shown
    assert "✅" in captured.out  # for the row with progress = True
    assert "❌" in captured.out  # for the row with progress = False

    # Ensure the correct columns exist in the output (check that the headers contain the expected keys)
    headers = ["key", "summary", "progress"]
    for header in headers:
        assert f"| {header} |" in captured.out  # Check that each header appears

    # Check that the rows have the correct values
    assert "| ✅" in captured.out
    assert "| ❌" in captured.out


@pytest.mark.timeout(1)  # Timeout after 1 second for safety
def test_lint_all_all_pass(mock_save_cache, cli, capsys):
    cli.jira = MagicMock()

    # Mock the AI provider (if used in validation)
    cli.ai_provider = MagicMock()
    cli.ai_provider.improve_text.return_value = "OK"

    # Mock list of issues
    cli.jira.list_issues.return_value = [
        {
            "key": "AAP-test_lint_all_all_pass-1",
            "fields": {
                "issuetype": {"name": "Story"},
                "status": {"name": "Refinement"},
                "reporter": None,
            },
        },
        {
            "key": "AAP-test_lint_all_all_pass-2",
            "fields": {
                "issuetype": {"name": "Story"},
                "status": {"name": "Refinement"},
                "reporter": None,
            },
        },
    ]

    # Mock the request function to return the issue details
    def mock_request(method, path, **kwargs):
        return {
            "fields": {
                "summary": "OK",
                "description": "OK",
                "priority": {"name": "High"},
                EnvFetcher.get("JIRA_STORY_POINTS_FIELD"): 5,
                EnvFetcher.get("JIRA_BLOCKED_FIELD"): {"value": "False"},
                EnvFetcher.get("JIRA_BLOCKED_REASON_FIELD"): "",
                "status": {"name": "Refinement"},  # Status is "Refinement"
                "assignee": {"displayName": "Someone"},
                EnvFetcher.get(
                    "JIRA_EPIC_FIELD"
                ): "AAP-test_lint_all_all_pass-3",  # No Epic assigned for Story issues with Refinement status
                "reporter": None,
            }
        }

    cli.jira._request = mock_request

    # Ensure the Args object has the required 'project' and other attributes
    class Args1:
        project = "TestProject"
        component = "analytics-hcc-service"
        reporter = None
        assignee = None

    # Patch validate where it's imported (in the lint_all module, not edit_issue)
    with patch(
        "commands.cli_lint_all.validate", return_value=[[], []]
    ):  # Correct patch for the validate function used in lint_all
        cli.lint_all(Args1())

        # Capture and print output
        captured = capsys.readouterr()
        print(f"Captured Output:\n{captured.out}")

        # Check assertions: we expect all issues to pass lint checks
        assert "✅ AAP-test_lint_all_all_pass-1 OK passed" in captured.out
        assert "✅ AAP-test_lint_all_all_pass-2 OK passed" in captured.out

    # Ensure the Args object has the required 'project' and other attributes
    class Args2:
        project = "TestProject"
        component = "analytics-hcc-service"
        reporter = "John"
        assignee = None

    # Patch validate where it's imported (in the lint_all module, not edit_issue)
    with patch(
        "commands.cli_lint_all.validate", return_value=[[], []]
    ):  # Correct patch for the validate function used in lint_all
        cli.lint_all(Args2())

        # Capture and print output
        captured = capsys.readouterr()
        print(f"Captured Output:\n{captured.out}")

        # Check assertions: we expect all issues to pass lint checks
        assert "✅ AAP-test_lint_all_all_pass-1 OK passed" in captured.out
        assert "✅ AAP-test_lint_all_all_pass-2 OK passed" in captured.out


def test_lint_all_no_issues(mock_save_cache, cli, capsys):
    cli.jira = MagicMock()
    cli.jira.ai_provider = MagicMock()

    cli.jira.list_issues.return_value = []

    cli.lint_all(Args())
    out = capsys.readouterr().out

    assert "✅ No issues assigned to you." in out

    cli.lint_all(ArgsReporter())
    out = capsys.readouterr().out

    assert "✅ No issues assigned to you." in out

    cli.lint_all(ArgsAssignee())
    out = capsys.readouterr().out

    assert "✅ No issues assigned to you." in out


def test_lint_all_exception(mock_save_cache, cli, capsys):
    cli.jira = MagicMock()
    cli.jira.ai_provider = MagicMock()

    cli.jira.list_issues.side_effect = LintAllError("Simulated failure")

    with pytest.raises(LintAllError):
        cli.lint_all(Args())
    out = capsys.readouterr().out

    assert "❌ Failed to lint issues: Simulated failure" in out


def test_lint_all_with_failures(mock_save_cache, cli, capsys):
    cli.jira = MagicMock()

    # Mock the AI provider (if used in validation)
    cli.ai_provider = MagicMock()
    cli.ai_provider.improve_text.return_value = "OK"

    # Mock list of issues
    # /* jscpd:ignore-start */
    cli.jira.list_issues.return_value = [
        {
            "key": "AAP-test_lint_all_with_failures-1",
            "fields": {
                "key": "AAP-test_lint_all_with_failures-1",
                "issuetype": {"name": "Story"},
                "status": {"name": "Refinement"},
                "reporter": None,
            },
        },
        {
            "key": "AAP-test_lint_all_with_failures-2",
            "fields": {
                "key": "AAP-test_lint_all_with_failures-2",
                "issuetype": {"name": "Story"},
                "status": {"name": "Refinement"},
                "reporter": None,
            },
        },
    ]

    # Mock the request function to return the issue details
    def mock_request(method, path, **kwargs):
        return {
            "fields": {
                "summary": "OK",
                "description": "OK",
                "priority": {"name": "High"},
                EnvFetcher.get("JIRA_STORY_POINTS_FIELD"): 5,
                EnvFetcher.get("JIRA_BLOCKED_FIELD"): {"value": "False"},
                EnvFetcher.get("JIRA_BLOCKED_REASON_FIELD"): "",
                "status": {"name": "Refinement"},  # Status is "Refinement"
                "assignee": {"displayName": "Someone"},
                EnvFetcher.get(
                    "JIRA_EPIC_FIELD"
                ): None,  # No Epic assigned for Story issues with Refinement status
                "reporter": None,
            }
        }

    # /* jscpd:ignore-end */

    cli.jira._request = mock_request

    # Patch validate to return problems
    with patch(
        "commands.cli_lint_all.validate",
        return_value=[["❌ Issue has no assigned Epic"], []],
    ):
        cli.lint_all(Args())

        # Capture and print output
        captured = capsys.readouterr()
        print(f"Captured Output:\n{captured.out}")

        # Assert that the lint check failure output is captured
        assert (
            "❌ AAP-test_lint_all_with_failures-1 OK failed lint checks" in captured.out
        )
        assert (
            "❌ AAP-test_lint_all_with_failures-2 OK failed lint checks" in captured.out
        )
        assert "⚠️ Issues with lint problems:" in captured.out
        assert "🔍 AAP-test_lint_all_with_failures-1 - OK" in captured.out
        assert " - ❌ Issue has no assigned Epic" in captured.out



from core.env_fetcher import EnvFetcher


def set_sprint(request_fn, issue_key, sprint_id):
    payload = {
        "fields": {
            EnvFetcher.get("JIRA_SPRINT_FIELD"): (
                None if not sprint_id else [str(sprint_id)]
            )
        }
    }

    request_fn(
        "PUT",
        f"/rest/api/2/issue/{issue_key}",
        json=payload,
    )



from unittest.mock import MagicMock

import pytest
from exceptions.exceptions import AssignIssueError
from rest.ops.assign_issue import assign_issue


def test_assign_issue_success():
    mock_request = MagicMock()
    result = assign_issue(mock_request, "ABC-123", "johndoe")

    # Verify function returns True
    assert result is True

    args, kwargs = mock_request.call_args
    assert args == ("PUT", "/rest/api/2/issue/ABC-123")
    assert kwargs["json"] == {"fields": {"assignee": {"name": "johndoe"}}}


def test_assign_issue_failure(capsys, client):
    client._request = MagicMock(side_effect=AssignIssueError("fail"))

    with pytest.raises(AssignIssueError):
        client.assign_issue("ABC-123", "johndoe")

    capsys, _ = capsys.readouterr()
    assert "❌ Failed to assign issue ABC-123" in capsys



from exceptions.exceptions import RemoveFromSprintError


def remove_from_sprint(request_fn, issue_key):
    try:
        request_fn(
            "POST",
            "/rest/agile/1.0/backlog/issue",
            json={"issues": [issue_key]},
        )
        print(f"✅ Moved {issue_key} to backlog")
    except RemoveFromSprintError as e:
        msg = f"❌ Failed to remove from sprint: {e}"
        print(msg)
        raise (RemoveFromSprintError(msg))



from exceptions.exceptions import UnBlockError


def cli_unblock(jira, args):
    try:
        jira.unblock_issue(args.issue_key)
        print(f"✅ {args.issue_key} marked as unblocked")
        return True
    except UnBlockError as e:
        msg = f"❌ Failed to unblock {args.issue_key}: {e}"
        print(msg)
        raise (UnBlockError(msg))



from core.env_fetcher import EnvFetcher


def test_unblock_issue_calls_expected_fields(client):
    called = {}

    def fake_request(method, path, json=None, **kwargs):
        called["method"] = method
        called["path"] = path
        called["json"] = json
        return {}

    client._request = fake_request

    client.unblock_issue("AAP-test_unblock_issue_calls_expected_fields")

    assert called["method"] == "PUT"
    assert (
        called["path"]
        == "/rest/api/2/issue/AAP-test_unblock_issue_calls_expected_fields"
    )
    assert called["json"] == {
        "fields": {
            EnvFetcher.get("JIRA_BLOCKED_FIELD"): {"value": "False"},
            EnvFetcher.get("JIRA_BLOCKED_REASON_FIELD"): "",
        }
    }



from unittest.mock import MagicMock

import pytest
from exceptions.exceptions import MigrateError


def test_migrate_to_exception(cli, capsys):
    # Mock the migrate_issue method to raise an exception
    cli.jira.migrate_issue = MagicMock(side_effect=MigrateError("fail"))
    cli.jira.jira_url = "http://fake"

    # Mock the Args class with necessary attributes
    class Args:
        issue_key = "AAP-test_migrate_to_exception"
        new_type = "story"

    with pytest.raises(MigrateError):
        # Call the migrate method
        cli.migrate(Args())

    # Capture the output and assert the error message
    out = capsys.readouterr().out
    assert "❌ Migration failed" in out



def migrate_issue(request_fn, jira_url, build_payload_fn, old_key, new_type):
    fields = request_fn("GET", f"/rest/api/2/issue/{old_key}")["fields"]
    summary = fields.get("summary", f"Migrated from {old_key}")
    description = fields.get("description", f"Migrated from {old_key}")

    payload = build_payload_fn(summary, description, new_type)
    new_key = request_fn("POST", "/rest/api/2/issue/", json=payload)["key"]

    request_fn(
        "POST",
        f"/rest/api/2/issue/{old_key}/comment",
        json={
            "body": f"Migrated to [{new_key}]({jira_url}/browse/{new_key}) as a {new_type.upper()}."
        },
    )

    transitions = request_fn("GET", f"/rest/api/2/issue/{old_key}/transitions")[
        "transitions"
    ]
    transition_id = next(
        (
            t["id"]
            for t in transitions
            if t["name"].lower() in ["done", "closed", "cancelled"]
        ),
        None,
    )
    if not transition_id and transitions:
        transition_id = transitions[0]["id"]

    if transition_id:
        request_fn(
            "POST",
            f"/rest/api/2/issue/{old_key}/transitions",
            json={"transition": {"id": transition_id}},
        )

    return new_key



import os
import subprocess

from exceptions.exceptions import OpenIssueError


def cli_open_issue(args):
    try:
        subprocess.Popen(
            ["xdg-open", os.getenv("JIRA_URL") + "/browse/" + args.issue_key]
        )
        return True
    except OpenIssueError as e:
        msg = f"❌ Failed to open issue {args.issue_key}: {e}"
        print(msg)
        raise (OpenIssueError(msg))



from exceptions.exceptions import AddSprintError


def cli_add_sprint(jira, args):
    try:
        jira.add_to_sprint_by_name(args.issue_key, args.sprint_name)
        print(f"✅ Added to sprint '{args.sprint_name}'")
        return True
    except AddSprintError as e:
        msg = f"❌ {e}"
        print(msg)
        raise (AddSprintError(msg))



from unittest.mock import MagicMock

from core.env_fetcher import EnvFetcher


def test_lint_data_structure(client):
    issue_data = {
        "fields": {
            "summary": "",
            "description": None,
            "priority": None,
            EnvFetcher.get("JIRA_STORY_POINTS_FIELD"): None,  # Story points
            EnvFetcher.get("JIRA_BLOCKED_FIELD"): {"value": "True"},  # Blocked
            EnvFetcher.get("JIRA_BLOCKED_REASON_FIELD"): "",  # Blocked reason
            "status": {"name": "In Progress"},
            "assignee": None,
        }
    }

    client._request = MagicMock(return_value=issue_data)
    result = client._request("GET", "/rest/api/2/issue/AAP-test_lint_data_structure")

    assert result["fields"]["status"]["name"] == "In Progress"
    assert result["fields"][EnvFetcher.get("JIRA_BLOCKED_FIELD")]["value"] == "True"



from exceptions.exceptions import ChangeIssueTypeError


def change_issue_type(request_fn, issue_key, new_type):
    try:
        issue_data = request_fn("GET", f"/rest/api/2/issue/{issue_key}")
        is_subtask = issue_data["fields"]["issuetype"]["subtask"]
        payload = {"fields": {"issuetype": {"name": new_type.capitalize()}}}
        if is_subtask:
            payload["update"] = {"parent": [{"remove": {}}]}

        request_fn("PUT", f"/rest/api/2/issue/{issue_key}", json=payload)
    except ChangeIssueTypeError as e:
        msg = f"❌ Failed to change issue type: {e}"
        print(msg)
        raise (ChangeIssueTypeError(msg))



from unittest.mock import MagicMock

import pytest
from exceptions.exceptions import ListIssuesError


def test_list_issues_empty(cli, capsys):
    # Mock list_issues to return an empty list
    cli.jira.list_issues = MagicMock(return_value=[])

    class Args:
        project = None
        component = None
        user = None
        assignee = None
        reporter = None

    cli.list_issues(Args())
    out = capsys.readouterr().out
    assert "No issues found." in out


def test_list_issues_fail(cli, capsys):
    # Mock list_issues to raise an exception
    cli.jira.list_issues = MagicMock(side_effect=ListIssuesError("fail"))

    class Args:
        project = None
        component = None
        assignee = None
        reporter = None

    with pytest.raises(ListIssuesError):
        cli.list_issues(Args())

    out = capsys.readouterr().out
    assert "❌ Failed to list issues" in out



from unittest.mock import MagicMock

import pytest


def test_add_to_sprint_by_name_success(client):
    # Mock the _request method to simulate sprint lookup and assignment
    client._request = MagicMock(
        side_effect=[
            {"values": [{"id": 88, "name": "Sprint 42"}]},  # Sprint lookup
            {},  # Assignment
        ]
    )

    # Call the add_to_sprint_by_name method
    client.add_to_sprint_by_name("AAP-test_add_to_sprint_by_name_success", "Sprint 42")

    # Assert that _request was called twice
    assert client._request.call_count == 2


def test_add_to_sprint_by_name_not_found(client):
    # Mock the _request method to simulate sprint lookup where the sprint is not found
    client._request = MagicMock(return_value={"values": []})

    # Try to add the issue to a non-existent sprint and check for the exception
    with pytest.raises(Exception) as exc:
        client.add_to_sprint_by_name(
            "AAP-test_add_to_sprint_by_name_not_found", "Nonexistent Sprint"
        )

    # Assert that the exception message contains 'Could not find sprint'
    assert "Could not find sprint" in str(exc.value)



import os

from core.env_fetcher import EnvFetcher


def unblock_issue(request_fn, issue_key):
    blocked_field = os.getenv(
        "JIRA_BLOCKED_FIELD", EnvFetcher.get("JIRA_BLOCKED_FIELD")
    )
    reason_field = os.getenv(
        "JIRA_BLOCKED_REASON_FIELD", EnvFetcher.get("JIRA_BLOCKED_REASON_FIELD")
    )

    payload = {
        "fields": {
            blocked_field: {"value": "False"},
            reason_field: "",  # Clear reason
        }
    }

    request_fn(
        "PUT",
        f"/rest/api/2/issue/{issue_key}",
        json=payload,
    )



import sys
from io import StringIO
from unittest.mock import MagicMock


def test_set_priority_love_input(cli):
    cli.jira = MagicMock()

    class Args:
        issue_key = "AAP-test_set_priority_love_input"
        priority = "me love you long time"

    captured_output = StringIO()
    sys.stdout = captured_output

    cli.set_priority(Args())

    sys.stdout = sys.__stdout__
    out = captured_output.getvalue()

    assert "✅ Priority set to 'me love you long time'" in out



def search_users(request_fn, query: str, max_results: int = 10) -> list:
    return request_fn(
        "GET",
        "/rest/api/2/user/search",
        params={"username": query, "maxResults": max_results},
    )



from unittest.mock import MagicMock


def test_add_comment_with_text(cli, capsys):
    # Mock dependencies using MagicMock
    cli.jira = MagicMock()
    cli.jira.add_comment = MagicMock()

    cli.ai_provider = MagicMock()
    cli.ai_provider.improve_text = MagicMock(return_value="Cleaned")

    class Args:
        issue_key = "AAP-test_add_comment_with_text"
        text = "Raw comment"

    cli.add_comment(Args())

    cli.jira.add_comment.assert_called_once_with(
        "AAP-test_add_comment_with_text", "Cleaned"
    )
    out = capsys.readouterr().out
    assert "✅ Comment added" in out



import tempfile
from unittest.mock import MagicMock, patch

import pytest
from exceptions.exceptions import AiError


def test_add_comment_editor(cli):
    # Mock the add_comment method and the improve_text method
    cli.jira.add_comment = MagicMock()
    cli.ai_provider.improve_text = MagicMock(return_value="my comment")

    # Create a temporary file and write to it
    tf = tempfile.NamedTemporaryFile(delete=False, mode="w+")
    tf.write("my comment")
    tf.flush()
    tf.seek(0)

    # Use the temporary file as input for the comment
    class Args:
        issue_key = "AAP-test_add_comment_editor"
        text = tf.name  # Use the file path for the comment

    # Call the add_comment method
    cli.add_comment(Args())

    # Clean up the temporary file
    # os.remove(tf.name)

    # Ensure the add_comment method was called
    cli.jira.add_comment.assert_called_once_with(
        "AAP-test_add_comment_editor", "my comment"
    )


def test_add_comment_with_editor_and_ai_exception_handling(cli, capsys):
    # Mock the AI provider's improve_text method to avoid calling the real AI service
    cli.ai_provider = MagicMock()
    cli.ai_provider.improve_text.side_effect = AiError("AI service failed")

    # Mock the add_comment method (to skip actual Jira interaction)
    cli.jira.add_comment = MagicMock()

    # Mock subprocess.call to avoid opening an editor
    with patch("subprocess.call") as mock_subprocess:
        # Mock TemplateLoader to avoid file access and slow processing
        with patch("builtins.input", return_value="test_input"):
            # Mock the tempfile.NamedTemporaryFile method to simulate file creation and reading
            with patch("tempfile.NamedTemporaryFile") as mock_tempfile:
                mock_tempfile.return_value.__enter__.return_value.write = MagicMock()
                mock_tempfile.return_value.__enter__.return_value.flush = MagicMock()
                mock_tempfile.return_value.__enter__.return_value.read = MagicMock(
                    return_value="Mocked comment"
                )

                # Create an empty text argument to trigger the else block
                class Args:
                    issue_key = (
                        "AAP-test_add_comment_with_editor_and_ai_exception_handling"
                    )
                    text = (
                        ""  # Empty text should trigger the else block (temporary file)
                    )

                with pytest.raises(AiError):
                    # Call the add_comment method
                    cli.add_comment(Args())

                # Capture the printed output
                captured = capsys.readouterr()

                # Check if the subprocess call was made (indicating editor use)
                mock_subprocess.assert_called_once()

                # Assert the expected output (you can check if the process was handled correctly)
                assert "AI service failed" in captured.out



from unittest.mock import MagicMock


def test_search_users_returns_expected_data(client):
    mock_users = [
        {"name": "daoneill", "displayName": "David O'Neill"},
        {"name": "jdoe", "displayName": "John Doe"},
    ]
    client._request = MagicMock(return_value=mock_users)

    results = client.search_users("daoneill")

    assert results == mock_users
    client._request.assert_called_once_with(
        "GET",
        "/rest/api/2/user/search",
        params={"username": "daoneill", "maxResults": 10},
    )



from ._try_cleanup import _try_cleanup  # noqa
from .cli_add_comment import cli_add_comment  # noqa
from .cli_add_sprint import cli_add_sprint  # noqa
from .cli_ai_helper import cli_ai_helper  # noqa
from .cli_assign import cli_assign  # noqa
from .cli_block import cli_block  # noqa
from .cli_blocked import cli_blocked  # noqa
from .cli_change_type import cli_change_type  # noqa
from .cli_create_issue import cli_create_issue  # noqa
from .cli_edit_issue import cli_edit_issue  # noqa
from .cli_lint import cli_lint  # noqa
from .cli_lint_all import cli_lint_all  # noqa
from .cli_list_issues import cli_list_issues  # noqa
from .cli_migrate import cli_migrate  # noqa
from .cli_open_issue import cli_open_issue  # noqa
from .cli_quarterly_connection import cli_quarterly_connection  # noqa
from .cli_remove_sprint import cli_remove_sprint  # noqa
from .cli_search import cli_search  # noqa
from .cli_search_users import cli_search_users  # noqa
from .cli_set_acceptance_criteria import cli_set_acceptance_criteria  # noqa
from .cli_set_priority import cli_set_priority  # noqa
from .cli_set_status import cli_set_status  # noqa
from .cli_set_story_epic import cli_set_story_epic  # noqa
from .cli_set_story_points import cli_set_story_points  # noqa
from .cli_talk import cli_talk  # noqa
from .cli_unassign import cli_unassign  # noqa
from .cli_unblock import cli_unblock  # noqa
from .cli_validate_issue import cli_validate_issue  # noqa
from .cli_view_issue import cli_view_issue  # noqa
from .cli_view_user import cli_view_user  # noqa
from .cli_vote_story_points import cli_vote_story_points  # noqa



from unittest.mock import MagicMock

import pytest
from exceptions.exceptions import GetUserError


def test_cli_view_user_prints_user_fields(cli, capsys):
    cli.jira.get_user = MagicMock()
    cli.jira.get_user.return_value = {
        "accountId": "abc123",
        "displayName": "David O'Neill",
        "emailAddress": "daoneill@redhat.com",
    }

    class Args:
        account_id = "abc123"

    cli.view_user(Args())

    out = capsys.readouterr().out
    assert "accountId : abc123" in out
    assert "displayName : David O'Neill" in out
    assert "emailAddress : daoneill@redhat.com" in out


def test_cli_view_user_raises_and_prints_error(cli, capsys):
    cli.jira.get_user = MagicMock()
    cli.jira.get_user.side_effect = GetUserError("User not found")

    class Args:
        account_id = "notreal"

    with pytest.raises(GetUserError) as e:
        cli.view_user(Args())

    out = capsys.readouterr().out
    assert "❌ Unable to retrieve user: User not found" in out
    assert "User not found" in str(e.value)



from unittest.mock import MagicMock

from core.env_fetcher import EnvFetcher


def test_block_issue_calls_expected_fields(client):
    client._request = MagicMock()
    client.block_issue("ABC-123", "Waiting for dependency")

    client._request.assert_called_once_with(
        "PUT",
        "/rest/api/2/issue/ABC-123",
        json={
            "fields": {
                EnvFetcher.get("JIRA_BLOCKED_FIELD"): {"value": "True"},
                EnvFetcher.get("JIRA_BLOCKED_REASON_FIELD"): "Waiting for dependency",
            }
        },
    )



import re

from core.env_fetcher import EnvFetcher
from exceptions.exceptions import SearchError


def cli_search(jira, args):
    try:
        jql = args.jql
        issues = jira.search_issues(jql)

        if issues is None or len(issues) == 0:
            print("❌ No issues found for the given JQL.")
            return False

        rows = []
        for issue in issues:
            f = issue["fields"]
            sprints = f.get(EnvFetcher.get("JIRA_SPRINT_FIELD")) or []
            sprint = next(
                (
                    re.search(r"name=([^,]+)", s).group(1)
                    for s in sprints
                    if "state=ACTIVE" in s and "name=" in s
                ),
                "—",
            )

            rows.append(
                (
                    issue["key"],
                    f["status"]["name"],
                    f["assignee"]["displayName"] if f["assignee"] else "Unassigned",
                    f.get("priority", {}).get("name", "—"),
                    str(f.get(EnvFetcher.get("JIRA_STORY_POINTS_FIELD"), "—")),
                    sprint,
                    f["summary"],
                )
            )

        rows.sort(key=lambda r: (r[5], r[1]))

        headers = [
            "Key",
            "Status",
            "Assignee",
            "Priority",
            "Points",
            "Sprint",
            "Summary",
        ]
        widths = [
            max(len(h), max(len(r[i]) for r in rows)) for i, h in enumerate(headers)
        ]

        header_fmt = " | ".join(h.ljust(w) for h, w in zip(headers, widths))
        print(header_fmt)
        print("-" * len(header_fmt))

        for r in rows:
            print(" | ".join(val.ljust(widths[i]) for i, val in enumerate(r)))

        return issues
    except SearchError as e:
        msg = f"❌ Failed to search issues: {e}"
        print(msg)
        raise (SearchError(msg))



def test_assign_success(cli, capsys):
    cli.jira.assign_issue = lambda k, a: True

    class Args:
        issue_key = "AAP-test_assign_success"
        assignee = "johndoe"

    cli.assign(Args())
    out = capsys.readouterr().out
    assert "✅ assigned AAP-test_assign_success to johndoe" in out


def test_assign_failure(cli, capsys):
    cli.jira.assign_issue = lambda k, a: False

    class Args:
        issue_key = "AAP-test_assign_failure"
        assignee = "johndoe"

    cli.assign(Args())
    out = capsys.readouterr().out
    assert "❌ Could not assign AAP-test_assign_failure to johndoe" in out



def test_rest_view_issue(client):
    # Call the method to set priority
    client.view_issue("AAP-test_rest_view_issue")

    # Update the test to expect the 'allow_204' argument
    client._request.assert_called_once_with(
        "GET",
        "/rest/api/2/issue/AAP-test_rest_view_issue",
    )



from unittest.mock import MagicMock

import pytest
from exceptions.exceptions import FetchIssueIDError, VoteStoryPointsError


def test_vote_story_points_success(client):
    # First call: get issue ID
    mock_issue_response = MagicMock()
    mock_issue_response.status_code = 200
    mock_issue_response.text = '{"id": "16775066"}'
    mock_issue_response.json.return_value = {"id": "16775066"}

    # Second call: vote
    mock_vote_response = MagicMock()
    mock_vote_response.status_code = 200
    mock_vote_response.text = '{"status": "ok"}'
    mock_vote_response.json.return_value = {"status": "ok"}

    client._request.side_effect = [mock_issue_response, mock_vote_response]

    client.vote_story_points("ISSUE-123", 3)

    # Assert the request was made twice
    assert client._request.call_count == 2


def test_vote_story_points_failure(client, capsys):
    # First call: get issue ID
    mock_issue_response = MagicMock()
    mock_issue_response.status_code = 200
    mock_issue_response.text = '{"id": "16775066"}'
    mock_issue_response.json.return_value = {"id": "16775066"}

    # Second call: vote fails
    mock_vote_response = MagicMock()
    mock_vote_response.status_code = 400
    mock_vote_response.text = '{"error": "bad request"}'

    client._request.side_effect = [mock_issue_response, mock_vote_response]

    with pytest.raises(VoteStoryPointsError):
        client.vote_story_points("ISSUE-123", 3)

    captured = capsys.readouterr()
    assert "❌ Failed to vote on story points: JIRA API error (400):" in captured.out


def test_vote_story_points_fetch_issue_id_failure(client, capsys):
    # Simulate the first request (GET issue) raising an exception
    client._request.side_effect = FetchIssueIDError("network error")

    with pytest.raises(FetchIssueIDError):
        client.vote_story_points("ISSUE-123", 3)

    captured = capsys.readouterr()
    assert "❌ Failed to fetch issue ID for ISSUE-123: network error" in captured.out



from commands.cli_validate_issue import (  # isort: skip # pylint: disable=E0611
    load_and_cache_issue,
    load_cache,
    save_cache,
)  # isort: skip


def test_mock_load_cache(mock_load_cache):
    result = load_cache()
    assert isinstance(result, dict)


def test_mock_save_cache(mock_save_cache):
    save_cache({"AAP-test_mock_save_cache": {"summary_hash": "data"}})
    # mock_save_cache.assert_called_once()


def test_mock_load_and_cache_issue(mock_load_and_cache_issue):
    result, _ = load_and_cache_issue("AAP-test_mock_save_cache")
    assert result == {"AAP-test_mock_save_cache": {"summary_hash": "data"}}



from core.env_fetcher import EnvFetcher


def set_story_epic(request_fn, issue_key, epic_key):
    request_fn(
        "PUT",
        f"/rest/api/2/issue/{issue_key}",
        json={"fields": {EnvFetcher.get("JIRA_EPIC_FIELD"): epic_key}},
    )



from unittest.mock import MagicMock

from core.env_fetcher import EnvFetcher


def test_set_story_points(client):
    client._request = MagicMock(return_value={})

    # Call the function to set story points
    client.set_story_points("AAP-test_set_story_points", 8)

    # Assert that the PUT request is called with the correct payload and endpoint
    client._request.assert_called_once_with(
        "PUT",
        "/rest/api/2/issue/AAP-test_set_story_points",
        json={"fields": {EnvFetcher.get("JIRA_STORY_POINTS_FIELD"): 8}},
    )



from unittest.mock import MagicMock


def test_get_user_returns_expected_data(client):
    mock_user = {
        "name": "daoneill",
        "displayName": "David O'Neill",
        "emailAddress": "daoneill@redhat.com",
    }
    client._request = MagicMock(return_value=mock_user)

    result = client.get_user("daoneill")

    assert result == mock_user
    client._request.assert_called_once_with(
        "GET", "/rest/api/2/user", params={"username": "daoneill"}
    )



from exceptions.exceptions import SetStatusError


def cli_set_status(jira, args):
    try:
        jira.set_status(args.issue_key, args.status)
        print(f"✅ Status set to '{args.status}'")
        return True
    except SetStatusError as e:
        msg = f"❌ Failed to update status: {e}"
        print(msg)
        raise (SetStatusError(msg))



from unittest.mock import MagicMock


def test_vote_story_points(cli, capsys):
    cli.jira.vote_story_points = MagicMock()

    class Args:
        issue_key = "AAP-test_vote_story_points"
        points = "8"

    cli.vote_story_points(Args())
    cli.jira.vote_story_points.assert_called_once_with("AAP-test_vote_story_points", 8)
    out = capsys.readouterr().out
    assert "✅ Voted" in out



from exceptions.exceptions import SetAcceptanceCriteriaError


def cli_set_acceptance_criteria(jira, args):
    try:
        jira.set_acceptance_criteria(args.issue_key, args.acceptance_criteria)
        print(f"✅ Acceptance criteria set to '{args.acceptance_criteria}'")
        return True
    except SetAcceptanceCriteriaError as e:
        msg = f"❌ Failed to set acceptance criteria: {e}"
        print(msg)
        raise (SetAcceptanceCriteriaError(msg))



from unittest.mock import MagicMock

import pytest
from core.env_fetcher import EnvFetcher
from exceptions.exceptions import LintError


def test_lint_command_flags_errors(mock_save_cache, cli, capsys):
    cli.ai_provider = MagicMock()
    cli.ai_provider.improve_text.side_effect = lambda prompt, text: (
        "too short" if text in ["Bad", "Meh"] else "OK"
    )

    fake_issue = {
        "fields": {
            "summary": "Bad",
            "description": "Meh",
            "priority": None,
            EnvFetcher.get("JIRA_STORY_POINTS_FIELD"): None,
            EnvFetcher.get("JIRA_BLOCKED_FIELD"): {"value": "True"},
            EnvFetcher.get("JIRA_BLOCKED_REASON_FIELD"): "",
            "status": {"name": "In Progress"},
            "assignee": None,
        }
    }

    cli.jira._request.return_value = fake_issue

    class Args:
        issue_key = "AAP-test_lint_command_flags_errors"

    cli.lint(Args())
    out = capsys.readouterr().out

    assert "⚠️ Lint issues found in AAP-test_lint_command_flags_errors" in out
    assert "❌ Summary: too short" in out
    assert "❌ Description: too short" in out
    assert "❌ Priority not set" in out
    assert "❌ Story points not assigned" in out
    assert "❌ Issue is blocked but has no blocked reason" in out
    assert "❌ Issue is In Progress but unassigned" in out


def test_lint_command_success(mock_save_cache, cli, capsys):
    cli.ai_provider = MagicMock()
    cli.ai_provider.improve_text.side_effect = lambda prompt, text: "OK"

    clean_issue = {
        "fields": {
            "summary": "Valid summary",
            "description": "All good",
            "priority": {"name": "Medium"},
            EnvFetcher.get("JIRA_STORY_POINTS_FIELD"): 5,
            EnvFetcher.get("JIRA_BLOCKED_FIELD"): {"value": "False"},
            EnvFetcher.get("JIRA_BLOCKED_REASON_FIELD"): "",
            "status": {"name": "To Do"},
            "assignee": {"displayName": "dev"},
            EnvFetcher.get("JIRA_EPIC_FIELD"): {
                "name": "Epic Name"
            },  # Add assigned Epic for a pass
        }
    }

    cli.jira._request.return_value = clean_issue

    class Args:
        issue_key = "AAP-test_lint_command_success"

    cli.lint(Args())
    out = capsys.readouterr().out
    assert "✅ AAP-test_lint_command_success passed all lint checks" in out


def test_lint_command_exception(mock_save_cache, cli, capsys):
    # ✅ Fix: Mock ai_provider on cli directly
    cli.ai_provider = MagicMock()
    cli.ai_provider.improve_text.side_effect = lambda prompt, text: "OK"
    cli.jira._request.side_effect = LintError("Simulated fetch failure")

    class Args:
        issue_key = "AAP-test_lint_command_exception"

    with pytest.raises(LintError):
        cli.lint(Args())

    out = capsys.readouterr().out
    assert (
        "❌ Failed to lint issue AAP-test_lint_command_exception: Simulated fetch failure"
        in out
    )



from unittest.mock import MagicMock

import pytest
from exceptions.exceptions import SearchUsersError


def test_cli_search_users_prints_results(cli, capsys):
    cli.jira.search_users = MagicMock()
    cli.jira.search_users.return_value = [
        {
            "name": "daoneill",
            "emailAddress": "daoneill@redhat.com",
            "displayName": "David O'Neill",
        }
    ]

    class Args:
        query = "daoneill"

    cli.search_users(Args())

    out = capsys.readouterr().out
    assert "🔹 User:" in out
    assert "name: daoneill" in out
    assert "displayName: David O'Neill" in out


def test_cli_search_users_prints_warning_on_empty(cli, capsys):
    cli.jira.search_users = MagicMock()
    cli.jira.search_users.return_value = []

    class Args:
        query = "unknown-user"

    cli.search_users(Args())

    out = capsys.readouterr().out
    assert "⚠️ No users found." in out


def test_cli_search_users_raises_on_error(cli, capsys):
    cli.jira.search_users = MagicMock(side_effect=SearchUsersError("API unreachable"))

    class Args:
        query = "error-trigger"

    with pytest.raises(SearchUsersError) as e:
        cli.search_users(Args())

    out = capsys.readouterr().out
    assert "❌ Unable to search users: API unreachable" in out
    assert "API unreachable" in str(e.value)



import json
import os
import subprocess
import tempfile

from exceptions.exceptions import AiError, CreateIssueError
from rest.prompts import IssueType, PromptLibrary
from templates.template_loader import TemplateLoader


def cli_create_issue(jira, ai_provider, default_prompt, template_dir, args):
    try:
        template = TemplateLoader(template_dir, args.type)
        fields = template.get_fields()
    except FileNotFoundError as e:
        print(f"Error: {e}")
        raise (FileNotFoundError(e))

    inputs = (
        {field: input(f"{field}: ") for field in fields}
        if not args.edit
        else {field: f"# {field}" for field in fields}
    )

    description = template.render_description(inputs)

    if args.edit is not None:
        with tempfile.NamedTemporaryFile(mode="w+", suffix=".tmp", delete=False) as tmp:
            tmp.write(description)
            tmp.flush()
            subprocess.call([os.environ.get("EDITOR", "vim"), tmp.name])
            tmp.seek(0)
            description = tmp.read()

    enum_type = IssueType[args.type.upper()]
    prompt = PromptLibrary.get_prompt(enum_type)

    try:
        description = ai_provider.improve_text(prompt, description)
    except AiError as e:
        msg = f"⚠️ AI cleanup failed. Using original text. Error: {e}"
        print(msg)
        raise (AiError(msg))

    payload = jira.build_payload(args.summary, description, args.type)

    if args.dry_run:
        print("📦 DRY RUN ENABLED")
        print("---- Description ----")
        print(description)
        print("---- Payload ----")
        print(json.dumps(payload, indent=2))
        return

    try:
        key = jira.create_issue(payload)
        print(f"✅ Created: {jira.jira_url}/browse/{key}")
        return key
    except CreateIssueError as e:
        msg = f"❌ Failed to create issue: {e}"
        print(msg)
        raise (CreateIssueError)



from exceptions.exceptions import SetStoryEpicError


def cli_set_story_epic(jira, args):
    try:
        jira.set_story_epic(args.issue_key, args.epic_key)
        print(f"✅ Story's epic set to '{args.epic_key}'")
        return True
    except SetStoryEpicError as e:
        msg = f"❌ Failed to set epic: {e}"
        print(msg)
        raise (SetStoryEpicError(msg))



from core.env_fetcher import EnvFetcher


def test_set_acceptance_criteria(capsys, client):
    issue_key = "AAP-test_set_acceptance_criteria"
    acceptance_criteria = "Acceptance criteria description"

    # Simulate the GET and PUT responses correctly
    client._request.side_effect = [
        {
            "fields": {
                EnvFetcher.get(
                    "JIRA_ACCEPTANCE_CRITERIA_FIELD"
                ): "Acceptance criteria description"
            }
        },  # GET response with 'fields'
        {},  # PUT response (successful)
    ]

    # Call the set_acceptance_criteria method
    client.set_acceptance_criteria(issue_key, acceptance_criteria)

    # Capture the output printed by the function
    captured = capsys.readouterr().out

    # Assert that the output contains the expected success message
    assert f"✅ Updated acceptance criteria of {issue_key}" in captured



# exceptions/jira_exceptions.py


class BaseException(Exception):
    """Base class for all Jira-related exceptions."""

    pass


class MissingConfigVariable(BaseException):
    """Exception raised when a required Jira environment variable is missing."""

    pass


class SetStoryEpicError(BaseException):
    pass


class ListIssuesError(BaseException):
    pass


class SetAcceptanceCriteriaError(BaseException):
    pass


class DispatcherError(BaseException):
    pass


class EditIssueError(BaseException):
    pass


class FetchDescriptionError(BaseException):
    pass


class EditDescriptionError(BaseException):
    pass


class RemoveFromSprintError(BaseException):
    pass


class ChangeIssueTypeError(BaseException):
    pass


class UnassignIssueError(BaseException):
    pass


class AssignIssueError(BaseException):
    pass


class FetchIssueIDError(BaseException):
    pass


class VoteStoryPointsError(BaseException):
    pass


class GetPromptError(BaseException):
    pass


class UpdateDescriptionError(BaseException):
    pass


class MigrateError(BaseException):
    pass


class OpenIssueError(BaseException):
    pass


class ViewIssueError(BaseException):
    pass


class AddSprintError(BaseException):
    pass


class SetStatusError(BaseException):
    pass


class BlockError(BaseException):
    pass


class UnBlockError(BaseException):
    pass


class AddCommentError(BaseException):
    pass


class AiError(BaseException):
    pass


class SearchError(BaseException):
    pass


class CreateIssueError(BaseException):
    pass


class LintAllError(BaseException):
    pass


class LintError(BaseException):
    pass


class SetPriorityError(BaseException):
    pass


class SetStoryPointsError(BaseException):
    pass


class ChangeTypeError(BaseException):
    pass


class ListBlockedError(BaseException):
    pass


class InvalidPromptError(BaseException):
    pass


class JiraClientRequestError(BaseException):
    pass


class QuarterlyConnectionError(BaseException):
    pass


class GTP4AllError(BaseException):
    pass


class AiProviderError(BaseException):
    pass


class AIHelperError(BaseException):
    pass


class GetUserError(BaseException):
    pass


class SearchUsersError(BaseException):
    pass



from unittest.mock import MagicMock, patch

import pytest
from providers.bart_provider import BARTProvider


def test_bart_provider_init():
    with patch.dict("os.environ", {"AI_URL": "http://test-bart-url"}):
        provider = BARTProvider()
        assert provider.url == "http://test-bart-url"
        assert provider.headers == {"Content-Type": "application/json"}


@patch("providers.bart_provider.requests.post")
def test_improve_text_success(mock_post):
    mock_response = MagicMock()
    mock_response.status_code = 200
    mock_response.json.return_value = {"output": "Improved text"}
    mock_post.return_value = mock_response

    provider = BARTProvider()
    result = provider.improve_text("Improve this", "Bad text")
    assert result == "Improved text"
    mock_post.assert_called_once()


@patch("providers.bart_provider.requests.post")
def test_improve_text_failure(mock_post):
    mock_response = MagicMock()
    mock_response.status_code = 500
    mock_response.text = "Internal Server Error"
    mock_post.return_value = mock_response

    provider = BARTProvider()
    with pytest.raises(
        Exception, match="BART request failed: 500 - Internal Server Error"
    ):
        provider.improve_text("Prompt", "Text")



from unittest.mock import MagicMock


def test_epic_field(client):
    # Mock the environment variable for the epic field
    client.build_payload = MagicMock(
        return_value={"fields": {"nonsense": "Epic Field"}}
    )

    # Call the method
    result = client.build_payload("summary", "description", "epic")

    # Assert that the expected epic field is present in the result
    assert "nonsense" in result["fields"]



import io
from unittest.mock import MagicMock, patch

import pytest
from core.env_fetcher import EnvFetcher
from exceptions.exceptions import SearchError


def test_search(cli, mock_search_issues):
    # Prepare the args object to simulate CLI arguments
    class Args:
        jql = "project = AAP AND status = 'In Progress'"
        assignee = None
        reporter = None

    # Mock stdout to capture printed output
    with patch("sys.stdout", new_callable=io.StringIO) as mock_stdout:
        cli.search(Args())

        # Capture the printed output
        captured_output = mock_stdout.getvalue()

        # Verify if the correct output is printed
        assert "AAP-mock_search_issues" in captured_output  # Issue key is printed
        assert "SaaS Sprint 2025-13" in captured_output  # Sprint name is printed
        assert "In Progress" in captured_output  # Status is printed
        assert "David O Neill" in captured_output  # Assignee name is printed


def test_search_no_issues(cli):
    # Mock search_issues to return an empty list of issues
    cli.jira.search_issues = MagicMock(return_value=[])

    # Prepare the args object to simulate CLI arguments
    class Args:
        jql = "project = AAP AND status = 'NonExistentStatus'"
        assignee = None
        reporter = None

    # Mock stdout to capture printed output
    with patch("sys.stdout", new_callable=io.StringIO) as mock_stdout:
        cli.search(Args())

        # Capture the printed output
        captured_output = mock_stdout.getvalue()

        # Verify that no issues found message is printed
        assert "❌ No issues found for the given JQL." in captured_output


def test_search_with_exception(cli):
    # Mock search_issues to raise an exception
    cli.jira.search_issues = MagicMock(side_effect=SearchError("An error occurred"))

    # Prepare the args object to simulate CLI arguments
    class Args:
        jql = "project = AAP AND status = 'NonExistentStatus'"
        assignee = None
        reporter = None

    # Mock stdout to capture printed output
    with patch("sys.stdout", new_callable=io.StringIO) as mock_stdout:
        with pytest.raises(SearchError):
            cli.search(Args())

        # Capture the printed output
        captured_output = mock_stdout.getvalue()

        # Verify that the error message is printed
        assert "❌ Failed to search issues: An error occurred" in captured_output


def test_list_with_summary_filter(cli, capsys):
    # Mock list_issues to return a list of issues
    cli.jira.list_issues.return_value = [
        {
            "key": "AAP-test_list_with_summary_filter-1",
            "fields": {
                "status": {"name": "In Progress"},
                "assignee": {"displayName": "Dino"},
                "priority": {"name": "High"},
                EnvFetcher.get("JIRA_STORY_POINTS_FIELD"): 5,
                EnvFetcher.get("JIRA_SPRINT_FIELD"): ["name=Spring, state=ACTIVE"],
                "summary": "Fix bugs",
            },
        },
        {
            "key": "AAP-test_list_with_summary_filter-2",
            "fields": {
                "status": {"name": "In Progress"},
                "assignee": {"displayName": "Alice"},
                "priority": {"name": "Low"},
                EnvFetcher.get("JIRA_STORY_POINTS_FIELD"): 3,
                EnvFetcher.get("JIRA_SPRINT_FIELD"): ["name=Summer, state=ACTIVE"],
                "summary": "Improve UX",
            },
        },
    ]

    # Mock the args with a summary filter
    args = type(
        "Args",
        (),
        {
            "project": None,
            "component": None,
            "user": None,
            "assignee": None,
            "reporter": None,
            "status": None,
            "summary": "Fix",  # Only issues with "Fix" in the summary should be shown
            "blocked": None,
            "unblocked": None,
        },
    )

    # Run the list method with the summary filter
    cli.list_issues(args)

    captured = capsys.readouterr()

    assert "AAP-test_list_with_summary_filter-1" in captured.out
    assert "AAP-test_list_with_summary_filter-2" not in captured.out



from unittest.mock import MagicMock


def test_migrate_success_print(cli):
    # Mock the migrate_issue method
    cli.jira.migrate_issue = MagicMock(return_value="AAP-test_migrate_success_print-0")
    cli.jira.jira_url = "http://fake"

    # Mock the Args class with necessary attributes
    class Args:
        issue_key = "AAP-test_migrate_success_print-1"
        new_type = "story"

    # Call the migrate method
    cli.migrate(Args())



from unittest.mock import MagicMock

import pytest
from exceptions.exceptions import SetStoryPointsError


def test_set_story_points_success(cli):
    mock_set_story_points = MagicMock()
    cli.jira = MagicMock(set_story_points=mock_set_story_points)

    class Args:
        issue_key = "AAP-test_set_story_points_success"
        points = 5

    cli.set_story_points(Args())
    mock_set_story_points.assert_called_once_with(
        "AAP-test_set_story_points_success", 5
    )


def test_set_story_points_failure(cli, capsys):
    def boom(issue_key, points):
        raise SetStoryPointsError("fake failure")

    cli.jira = MagicMock(set_story_points=boom)

    class Args:
        issue_key = "AAP-test_set_story_points_failure"
        points = 5

    with pytest.raises(SetStoryPointsError):
        cli.set_story_points(Args())

    captured = capsys.readouterr()
    assert "❌ Failed to set story points" in captured.out


def test_set_story_points_value_error(cli, capsys):
    class Args:
        issue_key = "AAP-test_set_story_points_value_error"
        points = "five"  # invalid non-integer value

    cli.set_story_points(Args())

    captured = capsys.readouterr()
    assert "❌ Points must be an integer." in captured.out



from argparse import Namespace
from unittest.mock import MagicMock, patch

import pytest
from exceptions.exceptions import AIHelperError

from commands.cli_ai_helper import (  # isort: skip
    ask_ai_question,
    clean_ai_output,
    get_cli_command_metadata,
    call_function,
)  # isort: skip


# Test Case: call_function - Verifies that the function dispatches the correct command
def test_call_function():
    # Mock the client and its _dispatch_command method
    mock_client = MagicMock()
    mock_client._dispatch_command = MagicMock()

    # Define the function name and arguments
    function_name = "test_command"
    args_dict = {"arg1": "value1", "arg2": "value2"}

    # Call the function
    call_function(mock_client, function_name, args_dict)

    # Create expected Namespace
    expected_args = Namespace(**args_dict)
    setattr(expected_args, "command", function_name)  # Required for _dispatch_command

    # Check that _dispatch_command was called with the correct args
    mock_client._dispatch_command.assert_called_once_with(expected_args)


def test_ai_helper_exec(cli):
    with pytest.raises(Exception):

        class Args:
            voice: bool

        cli.ai_helper(Args())


def test_get_cli_command_metadata_parses_correctly(cli):
    fake_action_positional = MagicMock()
    fake_action_positional.dest = "issue_key"
    fake_action_positional.required = True
    fake_action_positional.option_strings = []
    fake_action_positional.type = str
    fake_action_positional.help = "The issue key"

    fake_action_optional = MagicMock()
    fake_action_optional.dest = "status"
    fake_action_optional.required = False
    fake_action_optional.option_strings = ["--status"]
    fake_action_optional.type = str
    fake_action_optional.help = "Status to set"

    # Mocking subcommands with a mix of actions
    fake_subparser = MagicMock()
    fake_subparser.description = "Set issue status"
    fake_subparser._actions = [fake_action_positional, fake_action_optional]

    fake_subparsers = MagicMock()
    fake_subparsers.choices = {"set-status": fake_subparser}

    fake_parser = MagicMock()
    fake_parser.add_subparsers.return_value = fake_subparsers

    with patch("argparse.ArgumentParser", return_value=fake_parser):
        result = get_cli_command_metadata()

        assert "set-status" in result
        command = result["set-status"]

        # Assertions common to all tests
        assert command["help"] == "Set issue status"

        # Checking that only real arguments are included
        assert all(arg["name"] != "help" for arg in command["arguments"])
        assert command["arguments"][0]["name"] == "issue_key"
        assert command["arguments"][0]["positional"] is True

        assert command["arguments"][1]["name"] == "status"
        assert command["arguments"][1]["positional"] is False
        assert "--status" in command["arguments"][1]["flags"]


def test_clean_ai_output(cli):
    # Test valid JSON
    raw_valid = """```json
    [
        {"function": "set_status", "args": {"issue_key": "AAP-123", "status": "In Progress"}, "action": "test"}
    ]
    ```"""
    result = clean_ai_output(raw_valid)
    assert isinstance(result, list)
    assert result[0]["function"] == "set_status"

    # Test invalid JSON
    raw_invalid = "```json\nNot a JSON array\n```"
    with pytest.raises(ValueError) as exc_info:
        clean_ai_output(raw_invalid)
    assert "Failed to parse AI response" in str(exc_info.value)


# Test Case: cli_ai_helper - Verifies that AIHelperError is raised when get_cli_command_metadata throws an exception
def test_cli_ai_helper_exception(cli):
    # Mock get_cli_command_metadata to raise an exception
    with patch(
        "commands.cli_ai_helper.get_cli_command_metadata",
        side_effect=AIHelperError("Metadata fetching error"),
    ):
        with patch("commands.cli_ai_helper.ask_ai_question") as mock_ask_ai_question:
            with pytest.raises(AIHelperError) as exc_info:
                # Call the function with the mocks
                class Args:
                    voice: False
                    prompt: str

                args = Args()
                args.prompt = "Do something with issue"

                cli.ai_helper(args)

            # Check if AIHelperError was raised with the correct message
            assert "Failed to inspect public methods of JiraCLI" in str(exc_info.value)
            assert "Metadata fetching error" in str(exc_info.value)

            # Verify that ask_ai_question was not called because of the exception
            mock_ask_ai_question.assert_not_called()


# Test Case 1: AI generates an error message (returns a dictionary with "error" key)
@patch("commands.cli_ai_helper.gTTS")  # Mock gTTS to avoid audio generation
@patch("commands.cli_ai_helper.os.system")  # Mock os.system to prevent external calls
def test_ask_ai_question_error(mock_os_system, mock_gtts):
    mock_client = MagicMock()
    mock_ai_provider = MagicMock()
    mock_ai_provider.improve_text = MagicMock(
        return_value='{"error": "Something went wrong"}'
    )

    result = ask_ai_question(
        mock_client, mock_ai_provider, "system prompt", "user prompt", voice=True
    )

    assert result is False
    mock_os_system.assert_called_once_with("mpg123 output.mp3")
    mock_gtts.assert_called_once_with(text="Something went wrong", lang="en")


# Test Case 2: AI generates a non-error response (returns a dictionary without "error")
@patch("commands.cli_ai_helper.gTTS")  # Mock gTTS to avoid audio generation
@patch("commands.cli_ai_helper.os.system")  # Mock os.system to prevent external calls
def test_ask_ai_question_no_error(mock_os_system, mock_gtts):
    mock_client = MagicMock()
    mock_ai_provider = MagicMock()
    mock_ai_provider.improve_text = MagicMock(
        return_value='{"info": "Some steps to take"}'
    )

    result = ask_ai_question(
        mock_client, mock_ai_provider, "system prompt", "user prompt", voice=True
    )

    assert result is None
    mock_gtts.assert_not_called()  # No TTS should be triggered


# Test Case 3: AI generates a list of steps (returns a list of dictionaries)
@patch("commands.cli_ai_helper.gTTS")  # Mock gTTS to avoid audio generation
@patch("commands.cli_ai_helper.os.system")  # Mock os.system to prevent external calls
@patch(
    "commands.cli_ai_helper.call_function"
)  # Mock the call_function to avoid calling real functions
def test_ask_ai_question_steps(mock_call_function, mock_os_system, mock_gtts):
    mock_client = MagicMock()
    mock_ai_provider = MagicMock()
    mock_ai_provider.improve_text = MagicMock(
        return_value='[{"function": "function1", "args": {"arg1": "value1"}, "action": "test"}]'
    )

    result = ask_ai_question(
        mock_client, mock_ai_provider, "system prompt", "user prompt", voice=True
    )

    assert result is None
    mock_call_function.assert_called_once_with(
        mock_client, "function1", {"arg1": "value1"}
    )
    mock_os_system.assert_called_once_with("mpg123 output.mp3")


# Test Case 4: AI generates an empty list (returns an empty list)
@patch("commands.cli_ai_helper.gTTS")  # Mock gTTS to avoid audio generation
@patch("commands.cli_ai_helper.os.system")  # Mock os.system to prevent external calls
def test_ask_ai_question_empty_steps(mock_os_system, mock_gtts):
    mock_client = MagicMock()
    mock_ai_provider = MagicMock()
    mock_ai_provider.improve_text = MagicMock(return_value="[]")

    result = ask_ai_question(
        mock_client, mock_ai_provider, "system prompt", "user prompt", voice=True
    )

    assert result is False
    mock_gtts.assert_not_called()  # No TTS should be triggered


# Test Case 5: AI generates an invalid JSON (raises ValueError)
@patch("commands.cli_ai_helper.gTTS")  # Mock gTTS to avoid audio generation
@patch("commands.cli_ai_helper.os.system")  # Mock os.system to prevent external calls
def test_ask_ai_question_invalid_json(mock_os_system, mock_gtts):
    mock_client = MagicMock()
    mock_ai_provider = MagicMock()
    mock_ai_provider.improve_text = MagicMock(return_value="Invalid JSON")

    with pytest.raises(ValueError):
        ask_ai_question(
            mock_client, mock_ai_provider, "system prompt", "user prompt", voice=True
        )


# Test Case: Verifies that cli_ai_helper returns True when everything works correctly


@patch(
    "commands.cli_ai_helper.get_cli_command_metadata"
)  # Mock get_cli_command_metadata to return fake data
@patch(
    "commands.cli_ai_helper.ask_ai_question"
)  # Mock ask_ai_question to avoid actual AI processing
def test_cli_ai_helper_success(
    mock_ask_ai_question, mock_get_cli_command_metadata, cli
):
    # Setup mock return values for the dependencies
    mock_get_cli_command_metadata.return_value = {
        "command1": {
            "arguments": [{"name": "arg1", "positional": True, "help": "Help for arg1"}]
        },
        "command2": {
            "arguments": [
                {"name": "arg2", "positional": False, "help": "Help for arg2"}
            ]
        },
    }

    mock_args = MagicMock()  # Mock args
    mock_args.prompt = "Test prompt"
    mock_args.voice = True

    # Call the function
    result = cli.ai_helper(mock_args)

    # Verify that get_cli_command_metadata and ask_ai_question were called
    mock_get_cli_command_metadata.assert_called_once()
    mock_ask_ai_question.assert_called_once()

    # Check if the function returns True
    assert result is True



# conftest.py
from unittest.mock import MagicMock, patch

from rest.client import JiraClient  # pylint: disable=E0611
from rh_jira import JiraCLI

from core.env_fetcher import EnvFetcher  # isort: skip # pylint: disable=E0611

import pytest  # isort: skip

# Dummy file path and hash for testing
DUMMY_FILE_PATH = "/tmp/test_cache/ai-hashes.json"
DUMMY_HASH = "dummy_hash_value"


@pytest.fixture
def client():
    client = JiraClient()
    client._request = MagicMock()
    return client


# Fixture for patching subprocess.call
@pytest.fixture
def patch_subprocess_call():
    with patch(
        "commands.cli_edit_issue.subprocess.call", return_value=0
    ) as mock_subprocess:
        yield mock_subprocess


# Fixture for patching tempfile.NamedTemporaryFile
@pytest.fixture
def patch_tempfile_namedtemporaryfile():
    with patch("commands.cli_edit_issue.tempfile.NamedTemporaryFile") as mock_tempfile:
        # Mock tempfile behavior
        fake_file = MagicMock()
        fake_file.__enter__.return_value = fake_file
        fake_file.read.return_value = "edited content"
        fake_file.name = "/tmp/file.md"  # Using a fake file path
        mock_tempfile.return_value = fake_file
        yield mock_tempfile


# Fixture for CLI object
@pytest.fixture
def cli(
    patch_subprocess_call,
    patch_tempfile_namedtemporaryfile,
):
    # Apply the patches by simply referencing the patch fixtures
    patch_subprocess_call  # Applies patch to subprocess.call
    patch_tempfile_namedtemporaryfile  # Applies patch to tempfile.NamedTemporaryFile

    cli = JiraCLI()
    cli.jira = MagicMock()

    # Mock Jira methods
    cli.jira.get_description = MagicMock(return_value="Original description")
    cli.jira.update_description = MagicMock(return_value=True)
    cli.jira.get_issue_type = MagicMock(return_value="story")

    # Mock AI provider
    cli.ai_provider.improve_text = MagicMock(
        return_value="Cleaned and corrected content."
    )

    yield cli


# Mocking search_issues to return a list of issues
@pytest.fixture
def mock_search_issues(cli):
    # Mock search_issues to return a list of issues
    cli.jira.search_issues = MagicMock(
        return_value=[
            {
                "key": "AAP-mock_search_issues",
                "fields": {
                    "summary": "Run IQE tests in promotion pipelines",
                    "status": {"name": "In Progress"},
                    "assignee": {"displayName": "David O Neill"},
                    "priority": {"name": "Normal"},
                    EnvFetcher.get("JIRA_STORY_POINTS_FIELD"): 5,
                    EnvFetcher.get("JIRA_SPRINT_FIELD"): [
                        """com.atlassian.greenhopper.service.sprint.Sprint@5063ab17[id=70766,
                        rapidViewId=18242,state=ACTIVE,name=SaaS Sprint 2025-13,"
                        startDate=2025-03-27T12:01:00.000Z,endDate=2025-04-03T12:01:00.000Z]"""
                    ],
                },
            }
        ]
    )


# Mocking get_cache_path to return the dummy path
@pytest.fixture
def mock_cache_path():
    with patch(
        "commands.cli_validate_issue.get_cache_path",
        return_value=DUMMY_FILE_PATH,
    ):
        yield DUMMY_FILE_PATH


# Mocking load_cache to return a dummy cache
@pytest.fixture
def mock_load_cache(mock_cache_path):
    with patch(
        "commands.cli_validate_issue.load_cache",
        return_value={DUMMY_HASH: {"summary_hash": "dummy_summary_hash"}},
    ):
        yield


# Mocking save_cache to prevent actual file writing
@pytest.fixture
def mock_save_cache(mock_cache_path):
    with patch("commands.cli_validate_issue.save_cache") as mock_save:
        yield mock_save


# Mocking load_and_cache_issue to return a dummy cache and cached values
@pytest.fixture
def mock_load_and_cache_issue(mock_save_cache):
    data = (
        {"AAP-mock_load_and_cache_issue": {"summary_hash": DUMMY_HASH}},
        {"summary_hash": DUMMY_HASH},
    )
    with patch("commands.cli_validate_issue.load_and_cache_issue", return_value=data):
        yield



from core.env_fetcher import EnvFetcher


# /* jscpd:ignore-start */
def blocked(list_issues_fn, project=None, component=None, assignee=None):
    issues = list_issues_fn(project=project, component=component, assignee=assignee)

    blocked_issues = []
    for issue in issues:
        fields = issue["fields"]
        is_blocked = (
            fields.get(EnvFetcher.get("JIRA_BLOCKED_FIELD"), {}).get("value") == "True"
        )
        if is_blocked:
            blocked_issues.append(
                {
                    "key": issue["key"],
                    "status": fields["status"]["name"],
                    "assignee": (
                        fields["assignee"]["displayName"]
                        if fields["assignee"]
                        else "Unassigned"
                    ),
                    "reason": fields.get(
                        EnvFetcher.get("JIRA_BLOCKED_REASON_FIELD"), "(no reason)"
                    ),
                    "summary": fields["summary"],
                }
            )
    return blocked_issues


# /* jscpd:ignore-end */



def cli_assign(jira, args):
    success = jira.assign_issue(args.issue_key, args.assignee)
    print(
        f"✅ assigned {args.issue_key} to {args.assignee}"
        if success
        else f"❌ Could not assign {args.issue_key} to {args.assignee}"
    )
    return True if success else False



from unittest.mock import patch

import pytest
from exceptions.exceptions import OpenIssueError


def test_open_issue(cli):
    # Patch os.getenv to return a mock JIRA URL
    with patch("os.getenv", return_value="https://your-jira-url.com"):
        # Patch subprocess.Popen to prevent actually opening a process
        with patch("subprocess.Popen") as mock_popen:

            class Args:
                issue_key = "AAP-test_open_issue"

            # Simulate subprocess.Popen succeeding
            mock_popen.return_value = True

            # Call the method
            cli.open_issue(Args())

            # Assert that subprocess.Popen was called with the correct arguments
            mock_popen.assert_called_once_with(
                ["xdg-open", "https://your-jira-url.com/browse/AAP-test_open_issue"]
            )


def test_open_issue_exception_handling(cli):
    # Patch os.getenv to return a mock JIRA URL
    with patch("os.getenv", return_value="https://your-jira-url.com"):
        # Patch subprocess.Popen to simulate an exception
        with patch("subprocess.Popen") as mock_popen:

            class Args:
                issue_key = "AAP-test_open_issue_exception_handling"

            # Simulate subprocess.Popen raising an exception
            mock_popen.side_effect = OpenIssueError("Failed to open issue")

            # Call the method
            with patch(
                "builtins.print"
            ) as mock_print:  # Mock print to check the output
                with pytest.raises(OpenIssueError):
                    cli.open_issue(Args())

                # Assert that print was called with the correct error message
                mock_print.assert_called_once_with(
                    "❌ Failed to open issue AAP-test_open_issue_exception_handling: Failed to open issue"
                )



from exceptions.exceptions import ChangeTypeError


def cli_change_type(jira, args):
    try:
        if jira.change_issue_type(args.issue_key, args.new_type):
            print(f"✅ Changed {args.issue_key} to '{args.new_type}'")
            return True
        else:
            print(f"❌ Change failed for {args.issue_key}")
            return False
    except ChangeTypeError as e:
        msg = f"❌ Error: {e}"
        print(msg)
        raise (ChangeTypeError(msg))



def cli_unassign(jira, args):
    success = jira.unassign_issue(args.issue_key)
    print(
        f"✅ Unassigned {args.issue_key}"
        if success
        else f"❌ Could not unassign {args.issue_key}"
    )
    return True if success else False



import os
import subprocess
import tempfile
from pathlib import Path
from unittest.mock import MagicMock


def test_script_dry_run():
    # Set up environment
    env = os.environ.copy()
    env.update(
        {
            "JIRA_URL": "https://fake.jira",
            "PROJECT_KEY": "FAKE",
            "AFFECTS_VERSION": "0.0.1",
            "COMPONENT_NAME": "dummy-component",
            "PRIORITY": "Low",
            "JPAT": "fake-token",
            "AI_PROVIDER": "noop",
        }
    )

    # Create a temporary template file
    with tempfile.TemporaryDirectory() as tmpdir:
        template_dir = Path(tmpdir)
        template_path = template_dir / "bug.tmpl"
        template_path.write_text(
            "FIELD|Title\nTEMPLATE|Description\nBug Title: {{Title}}"
        )

        script_path = Path("jira_creator/rh_jira.py")

        # Mock subprocess.run to avoid actually running the script
        subprocess.run = MagicMock()

        # Call the script (you can use the mock here if you want to check the call)
        subprocess.run(
            [
                "python3",
                str(script_path),
                "--dry-run",
                "--template",
                str(template_path),
            ],
            env=env,
        )

        # Check if subprocess.run was called as expected
        subprocess.run.assert_called_once_with(
            [
                "python3",
                str(script_path),
                "--dry-run",
                "--template",
                str(template_path),
            ],
            env=env,
        )



from unittest.mock import MagicMock, patch

import pytest
from providers.openai_provider import OpenAIProvider


def test_openai_provider_improve_text():
    mock_response = type(
        "Response",
        (),
        {
            "status_code": 200,
            "json": lambda self: {
                "choices": [{"message": {"content": "Cleaned up text"}}]
            },
        },
    )()

    with patch("providers.openai_provider.requests.post", return_value=mock_response):
        provider = OpenAIProvider()
        result = provider.improve_text("fix this", "some bad text")
        assert result == "Cleaned up text"


def test_openai_provider_raises_without_api_key():
    with patch.dict("os.environ", {"AI_API_KEY": ""}):  # Set to an empty string
        with pytest.raises(
            EnvironmentError, match="AI_API_KEY not set in environment."
        ):
            OpenAIProvider()  # This should raise an EnvironmentError


def test_improve_text_raises_on_api_failure():
    provider = OpenAIProvider()
    provider.api_key = "fake-key"
    provider.model = "gpt-3.5-turbo"
    provider.endpoint = "https://api.openai.com/v1/chat/completions"

    mock_response = MagicMock()
    mock_response.status_code = 500
    mock_response.text = "Internal Server Error"

    with patch("providers.openai_provider.requests.post", return_value=mock_response):
        with pytest.raises(Exception) as exc_info:
            provider.improve_text("test prompt", "test input")

    assert "OpenAI API call failed: 500 - Internal Server Error" in str(exc_info.value)



import pytest


def test_set_status_valid_transition(client):
    # Mock response for GET and POST requests
    transitions = {"transitions": [{"name": "In Progress", "id": "31"}]}
    client._request.return_value = transitions  # First call is GET, second is POST

    # Call the set_status method
    client.set_status("AAP-test_set_status_valid_transition", "In Progress")

    # Assert that _request was called twice (GET and POST)
    assert client._request.call_count == 2


def test_set_status_invalid_transition(client):
    # Mock response for GET and POST requests
    transitions = {"transitions": [{"name": "In Progress", "id": "31"}]}
    client._request.return_value = transitions  # First call is GET, second is POST

    # Use pytest.raises to capture the exception
    with pytest.raises(Exception, match="❌ Transition to status 'Done' not found"):
        client.set_status("AAP-test_set_status_invalid_transition", "Done")

    # Ensure _request was called twice (GET and POST)
    assert client._request.call_count == 1



from unittest.mock import MagicMock


def test_add_sprint(cli):
    cli.jira.add_to_sprint_by_name = MagicMock()

    class Args:
        issue_key = "AAP-test_add_sprint"
        sprint_name = "Sprint 1"

    cli.add_sprint(Args())
    cli.jira.add_to_sprint_by_name.assert_called_once()


def test_remove_sprint(cli):
    cli.jira.remove_from_sprint = MagicMock()

    class Args:
        issue_key = "AAP-test_remove_sprint"

    cli.remove_sprint(Args())
    cli.jira.remove_from_sprint.assert_called_once()



from unittest.mock import MagicMock

import pytest
from exceptions.exceptions import SetStoryEpicError


def test_handle_success(cli, capsys):
    cli.jira.set_story_epic = MagicMock()

    class Args:
        issue_key = "AAP-test_handle_success"
        epic_key = "EPIC-123"

    # Call the handle function
    cli.set_story_epic(Args())

    # Capture the printed output
    captured = capsys.readouterr()

    # Assert that the correct message was printed
    assert "✅ Story's epic set to 'EPIC-123'" in captured.out

    # Ensure that set_story_epic was called with the correct arguments
    cli.jira.set_story_epic.assert_called_once_with(
        "AAP-test_handle_success", "EPIC-123"
    )


def test_set_story_epic_exception(cli, capsys):
    cli.jira.set_story_epic = MagicMock(side_effect=SetStoryEpicError("fail"))

    class Args:
        issue_key = "AAP-test_set_story_epic_exception"
        epic_key = "EPIC-123"
        reporter = "me"
        project = "aa"
        component = "bb"

    with pytest.raises(SetStoryEpicError):
        # Call the handle function
        cli.set_story_epic(Args())



from commands.cli_validate_issue import cli_validate_issue as validate
from exceptions.exceptions import LintError


def cli_lint(jira, ai_provider, args):
    try:
        issue = jira._request("GET", f"/rest/api/2/issue/{args.issue_key}")
        fields = issue["fields"]
        fields["key"] = args.issue_key

        problems = validate(fields, ai_provider)[0]

        if problems:
            print(f"⚠️ Lint issues found in {args.issue_key}:")
            for p in problems:
                print(f" - {p}")
            return problems
        else:
            print(f"✅ {args.issue_key} passed all lint checks")
            return problems
    except LintError as e:
        msg = f"❌ Failed to lint issue {args.issue_key}: {e}"
        print(msg)
        raise (LintError(msg))



import os
from enum import Enum


class IssueType(Enum):
    BUG = "bug"
    EPIC = "epic"
    SPIKE = "spike"
    STORY = "story"
    TASK = "task"
    COMMENT = "comment"
    DEFAULT = "default"
    QC = "qc"
    AIHELPER = "aihelper"


TEMPLATE_DIR = os.path.join(os.path.dirname(__file__), "../templates")


class PromptLibrary:
    @staticmethod
    def get_file_contents(full_name):
        template = ""
        template_path = os.path.join(TEMPLATE_DIR, f"{full_name}.tmpl")

        if not os.path.exists(template_path):
            raise FileNotFoundError(f"Template not found: {template_path}")

        with open(template_path, "r", encoding="utf-8") as f:
            template = f.read().strip()

        return template

    @staticmethod
    def get_prompt(issue_type: IssueType) -> str:
        # Check if the issue_type is "comment" first
        prompt = ""
        full_name = issue_type.value.lower()

        if issue_type == IssueType.DEFAULT:
            prompt = (
                PromptLibrary.get_file_contents("rules")
                + PromptLibrary.get_file_contents("base").format(type=issue_type.value)
                + PromptLibrary.get_file_contents(full_name)
            )
        elif issue_type == IssueType.COMMENT:
            prompt = PromptLibrary.get_file_contents(
                full_name
            ) + PromptLibrary.get_file_contents("rules")
        elif issue_type == IssueType.AIHELPER:
            prompt = PromptLibrary.get_file_contents(full_name)
        elif issue_type == IssueType.QC:
            prompt = PromptLibrary.get_file_contents(full_name)
        elif issue_type in [issue_type for issue_type in IssueType]:
            prompt = PromptLibrary.get_file_contents(
                "generic"
            ) + PromptLibrary.get_file_contents("rules")

        return prompt



from unittest.mock import MagicMock

import pytest
from exceptions.exceptions import AiError


def test_add_comment_ai_fail(cli, capsys):
    # Mock the add_comment method
    cli.jira.add_comment = MagicMock()

    # Mock ai_provider's improve_text method to raise an exception
    ai_provider_mock = MagicMock()
    ai_provider_mock.improve_text.side_effect = AiError("fail")
    cli.ai_provider = ai_provider_mock

    class Args:
        issue_key = "AAP-test_add_comment_ai_fail"
        text = "Comment text"

    with pytest.raises(AiError):
        # Call the method
        cli.add_comment(Args())

    # Capture output and assert
    out = capsys.readouterr().out
    assert "⚠️ AI cleanup failed" in out



def add_to_sprint_by_name(request_fn, board_id, issue_key, sprint_name):
    if not board_id:
        raise Exception("❌ JIRA_BOARD_ID not set in environment")

    sprints = request_fn("GET", f"/rest/agile/1.0/board/{board_id}/sprint").get(
        "values", []
    )
    sprint_id = next((s["id"] for s in sprints if s["name"] == sprint_name), None)

    if not sprint_id:
        raise Exception(f"❌ Could not find sprint named '{sprint_name}'")

    request_fn(
        "POST",
        f"/rest/agile/1.0/sprint/{sprint_id}/issue",
        json={"issues": [issue_key]},
    )
    print(f"✅ Added {issue_key} to sprint '{sprint_name}' on board {board_id}")



from exceptions.exceptions import GTP4AllError
from gpt4all import GPT4All


class GPT4AllProvider:
    def __init__(self, model_name: str = "ggml-gpt4all-j-v1.3-groovy"):
        self.model_name = model_name
        try:
            self.model = GPT4All(model_name)
        except GTP4AllError as e:
            raise GTP4AllError(RuntimeError(f"Failed to load GPT4All model: {e}"))

    def improve_text(self, prompt: str, text: str) -> str:
        instruction = (
            f"{prompt}\n\n"
            f"---\n"
            f"{text}\n"
            f"---\n\n"
            f"Please provide the improved version of the text, maintaining the structure."
        )
        response = self.model.generate(instruction, max_tokens=1024, temp=0.3)
        return response.strip()



from exceptions.exceptions import ViewIssueError


def cli_view_issue(jira, args):
    try:
        issue = jira.view_issue(args.issue_key)

        # Create a new dictionary with real names as keys
        updated_issue = {}

        for key in issue:
            # Check if the key is a custom field
            if "customfield" in key:
                real_name = jira.get_field_name(key)
                updated_issue[real_name] = issue[key]
            else:
                # For non-custom fields, keep the original key
                updated_issue[key] = issue[key]

        # Sort the dictionary by the real names (keys)
        for key in sorted(updated_issue.keys()):
            print(f"{key} : {updated_issue[key]}")

        return issue
    except ViewIssueError as e:
        msg = f"❌ Unable to view issue: {e}"
        print(msg)
        raise (ViewIssueError(msg))



from exceptions.exceptions import VoteStoryPointsError


def cli_vote_story_points(jira, args):
    try:
        points = int(args.points)
    except ValueError:
        print("❌ Points must be an integer.")
        return False

    try:
        jira.vote_story_points(args.issue_key, points)
        print(f"✅ Voted {points} points on {args.issue_key}")
        return True
    except VoteStoryPointsError as e:
        msg = f"❌ Failed to vote on story points: {e}"
        print(msg)
        raise (VoteStoryPointsError(msg))



from exceptions.exceptions import GetUserError


def cli_view_user(jira, args):
    try:
        user = jira.get_user(args.account_id)

        for key in sorted(user.keys()):
            print(f"{key} : {user[key]}")
        return user
    except GetUserError as e:
        msg = f"❌ Unable to retrieve user: {e}"
        print(msg)
        raise GetUserError(msg)



from unittest.mock import MagicMock

import pytest
from exceptions.exceptions import RemoveFromSprintError


def test_remove_from_sprint_error(capsys, client):
    # Mock the _request method to raise an exception
    client._request = MagicMock(side_effect=RemoveFromSprintError("fail"))

    with pytest.raises(RemoveFromSprintError):
        # Call the remove_from_sprint method
        client.remove_from_sprint("AAP-test_remove_from_sprint_error")

    # Capture the output and assert the error message
    out = capsys.readouterr().out
    assert "❌ Failed to remove from sprint" in out



def test_build_payload_with_patch_dict(client):
    summary = "Fix login issue"
    description = "Steps to reproduce..."
    issue_type = "bug"

    payload = client.build_payload(summary, description, issue_type)
    fields = payload["fields"]

    assert fields["project"]["key"] == "XYZ"
    assert fields["summary"] == summary
    assert fields["description"] == description
    assert fields["issuetype"]["name"] == "Bug"
    assert fields["priority"]["name"] == "High"
    assert fields["versions"][0]["name"] == "v1.2.3"
    assert fields["components"][0]["name"] == "backend"


# def test_missing_env_raises(client):
#     with pytest.raises(MissingConfigVariable):
#         JiraClient()



from unittest.mock import MagicMock

import pytest
from exceptions.exceptions import AiError


def test_try_cleanup_error(cli):
    # Mock the AI provider's improve_text method to simulate an exception
    cli.ai_provider = MagicMock()
    cli.ai_provider.improve_text.side_effect = AiError("fail")

    with pytest.raises(AiError):
        # Call _try_cleanup and assert the result
        cli._try_cleanup("prompt", "text")



from exceptions.exceptions import UnassignIssueError


def unassign_issue(request_fn, issue_key):
    try:
        request_fn(
            "PUT", f"/rest/api/2/issue/{issue_key}", json={"fields": {"assignee": None}}
        )
        return True
    except UnassignIssueError as e:
        msg = f"❌ Failed to unassign issue {issue_key}: {e}"
        print(msg)
        raise (UnassignIssueError(msg))



def test_set_status(client):
    # Simulating the side effects for multiple calls
    client._request.side_effect = [{"transitions": [{"name": "Done", "id": "2"}]}, {}]

    client.set_status("AAP-test_set_status", "Done")

    # Assert that the request was called twice
    assert client._request.call_count == 2



from .add_comment import add_comment  # noqa
from .add_to_sprint_by_name import add_to_sprint_by_name  # noqa
from .assign_issue import assign_issue  # noqa
from .block_issue import block_issue  # noqa
from .blocked import blocked  # noqa
from .build_payload import build_payload  # noqa
from .change_issue_type import change_issue_type  # noqa
from .create_issue import create_issue  # noqa
from .get_acceptance_criteria import get_acceptance_criteria  # noqa
from .get_current_user import get_current_user  # noqa
from .get_description import get_description  # noqa
from .get_issue_type import get_issue_type  # noqa
from .get_user import get_user  # noqa
from .list_issues import list_issues  # noqa
from .migrate_issue import migrate_issue  # noqa
from .remove_from_sprint import remove_from_sprint  # noqa
from .search_issues import search_issues  # noqa
from .search_users import search_users  # noqa
from .set_acceptance_criteria import set_acceptance_criteria  # noqa
from .set_priority import set_priority  # noqa
from .set_sprint import set_sprint  # noqa
from .set_status import set_status  # noqa
from .set_story_epic import set_story_epic  # noqa
from .set_story_points import set_story_points  # noqa
from .unassign_issue import unassign_issue  # noqa
from .unblock_issue import unblock_issue  # noqa
from .update_description import update_description  # noqa
from .view_issue import view_issue  # noqa
from .vote_story_points import vote_story_points  # noqa



from unittest.mock import MagicMock

import pytest
from exceptions.exceptions import SetStatusError


def test_set_status_exception(cli, capsys):
    # Mock the set_status method to simulate an exception
    cli.jira.set_status = MagicMock(side_effect=SetStatusError("bad status"))

    class Args:
        issue_key = "AAP-test_set_status_exception"
        status = "Invalid"

    with pytest.raises(SetStatusError):
        # Call the method
        cli.set_status(Args())

    # Capture the output
    out = capsys.readouterr().out

    # Assert that the correct error message was printed
    assert "❌ Failed to update status" in out



from unittest.mock import MagicMock, patch

from commands.cli_validate_issue import cli_validate_issue, load_cache, sha256
from core.env_fetcher import EnvFetcher


# Define a helper function for generating common fields
def generate_fields(
    issue_key,
    summary="Test Summary",
    description="Test Description",
    acceptance_criteria="Test Acceptance Criteria",
):
    return {
        "key": issue_key,
        "summary": summary,
        "description": description,
        EnvFetcher.get("JIRA_ACCEPTANCE_CRITERIA_FIELD"): acceptance_criteria,
        EnvFetcher.get("JIRA_EPIC_FIELD"): "Epic Link",
        "priority": {"name": "High"},
        EnvFetcher.get("JIRA_STORY_POINTS_FIELD"): 5,
        "status": {"name": "To Do"},
    }


# Define a helper function for generating cached data


def generate_cached_data(
    fields,
    description_hash=None,
    summary_hash=None,
    acceptance_criteria_hash=None,
    acceptance_criteria_value=None,
    description_value=None,
):
    if description_hash is None:
        description_hash = sha256(fields["description"])
    if summary_hash is None:
        summary_hash = sha256(fields["summary"])
    if acceptance_criteria_hash is None:
        acceptance_criteria_hash = sha256(
            fields[EnvFetcher.get("JIRA_ACCEPTANCE_CRITERIA_FIELD")]
        )

    # Ensure that description_value is used if passed
    description_value = description_value or "Needs Improvement"
    acceptance_criteria_value = acceptance_criteria_value or "Needs Improvement"

    return {
        "last_ai_acceptance_criteria": acceptance_criteria_value,
        "acceptance_criteria_hash": acceptance_criteria_hash,
        "last_ai_description": description_value,
        "description_hash": description_hash,
        "last_ai_summary": "Ok",
        "summary_hash": summary_hash,
    }


def test_load_cache_file_not_found():
    with patch("os.path.exists", return_value=False):
        result = load_cache()
        assert (
            result == {}
        ), "Expected an empty dictionary when the cache file doesn't exist"


def test_acceptance_criteria_no_change_but_invalid(mock_load_cache, mock_save_cache):
    ai_provider = MagicMock()
    ai_provider.improve_text.return_value = "Needs Improvement"

    fields = generate_fields(
        "AAP-test_acceptance_criteria_no_change_but_invalid",
        acceptance_criteria="Needs Improvement",
    )
    cached_data = generate_cached_data(fields)

    with patch("commands.cli_validate_issue.save_cache"):
        with patch(
            "commands.cli_validate_issue.load_cache",
            return_value={fields["key"]: cached_data},
        ):
            problems = cli_validate_issue(fields, ai_provider)[0]
            assert "❌ Acceptance Criteria: Needs Improvement" in problems
            assert (
                "❌ Acceptance Criteria: Check the quality of the following Jira acceptance criteria."
                not in problems
            )


def test_acceptance_criteria_validation(mock_save_cache, cli, capsys):
    ai_provider = MagicMock()
    ai_provider.improve_text.return_value = "OK"

    fields = generate_fields("AAP-test_acceptance_criteria_validation")

    with patch(
        "commands.cli_validate_issue.load_cache",
        return_value={fields["key"]: {"acceptance_criteria_hash": "old_hash"}},
    ):
        problems = cli_validate_issue(fields, ai_provider)[0]
        assert [] == problems


def test_description_no_change_but_invalid(mock_save_cache, cli, capsys):
    ai_provider = MagicMock()
    ai_provider.improve_text.return_value = "Needs Improvement"

    fields = generate_fields(
        "AAP-test_description_no_change_but_invalid", description="Needs Improvement"
    )
    cached_data = generate_cached_data(
        fields, acceptance_criteria_value="Needs Improvement"
    )

    with patch(
        "commands.cli_validate_issue.load_cache",
        return_value={fields["key"]: cached_data},
    ):
        with patch("commands.cli_validate_issue.save_cache") as _:
            problems = cli_validate_issue(fields, ai_provider)[0]
            # Now check for "Description" since we correctly set the description in the cached data
            assert "❌ Description: Needs Improvement" in problems
            assert (
                "❌ Description: Check the quality of the following Jira description."
                not in problems
            )


def test_cli_validate_issue(cli):
    class Args:
        issue_key = "AAP-test_edit_with_ai"
        no_ai = False
        lint = False

    cli.validate_issue({})



from unittest.mock import MagicMock


def test_migrate_no_transitions(client):
    def mock_request(method, path, **kwargs):
        if path.startswith(
            "/rest/api/2/issue/AAP-test_migrate_no_transitions/transitions"
        ):
            return {"transitions": []}
        elif path.startswith("/rest/api/2/issue/AAP-test_migrate_no_transitions"):
            return {"fields": {"summary": "Old", "description": "Old"}}
        elif path.startswith("/rest/api/2/issue/"):
            return {"key": "AAP-test_migrate_no_transitions"}

    client._request = MagicMock(side_effect=mock_request)
    client.jira_url = "http://fake"

    new_key = client.migrate_issue("AAP-test_migrate_no_transitions", "story")
    assert new_key == "AAP-test_migrate_no_transitions"



from unittest.mock import MagicMock


def test_get_update_description(client):
    # Mock _request method to simulate getting description
    client._request = MagicMock(return_value={"fields": {"description": "text"}})

    # Call get_description and assert it returns the correct description
    desc = client.get_description("AAP-test_get_update_description")
    assert desc == "text"

    # Create a dictionary to capture the updated fields
    updated = {}

    # Mock _request method to simulate updating description
    client._request = MagicMock(
        side_effect=lambda *a, **k: updated.update(k.get("json", {}))
    )

    # Call update_description and assert that the description field is updated
    client.update_description("AAP-test_get_update_description", "new text")
    assert "description" in updated["fields"]



#!/usr/bin/env python3
import os
import sys
from argparse import ArgumentParser, Namespace, _SubParsersAction
from pathlib import Path

from core.env_fetcher import EnvFetcher
from exceptions.exceptions import DispatcherError
from providers import get_ai_provider
from rest.client import JiraClient
from rest.prompts import IssueType, PromptLibrary

from commands import (  # isort: skip
    _try_cleanup,
    cli_add_comment,
    cli_add_sprint,
    cli_ai_helper,
    cli_assign,
    cli_block,
    cli_blocked,
    cli_change_type,
    cli_create_issue,
    cli_edit_issue,
    cli_lint,
    cli_lint_all,
    cli_list_issues,
    cli_open_issue,
    cli_migrate,
    cli_remove_sprint,
    cli_quarterly_connection,
    cli_search,
    cli_search_users,
    cli_set_acceptance_criteria,
    cli_set_priority,
    cli_set_status,
    cli_set_story_epic,
    cli_set_story_points,
    cli_talk,
    cli_unassign,
    cli_unblock,
    cli_validate_issue,
    cli_view_issue,
    cli_view_user,
    cli_vote_story_points,
)


class JiraCLI:
    def __init__(self) -> None:
        self.jira: JiraClient = JiraClient()
        required_vars: list[str] = [
            "JPAT",
            "AI_PROVIDER",
            "JIRA_URL",
            "PROJECT_KEY",
            "AFFECTS_VERSION",
            "COMPONENT_NAME",
            "PRIORITY",
            "AI_API_KEY",
            "JIRA_BOARD_ID",
            "JIRA_EPIC_FIELD",
            "JIRA_ACCEPTANCE_CRITERIA_FIELD",
            "JIRA_BLOCKED_FIELD",
            "JIRA_BLOCKED_REASON_FIELD",
            "JIRA_STORY_POINTS_FIELD",
            "JIRA_SPRINT_FIELD",
        ]
        EnvFetcher.fetch_all(required_vars)
        self.template_dir: Path = Path(
            os.getenv(
                "TEMPLATE_DIR", os.path.join(os.path.dirname(__file__), "templates")
            )
        )
        self.ai_provider = get_ai_provider(os.getenv("AI_PROVIDER", "openai"))
        self.default_prompt = PromptLibrary.get_prompt(IssueType["DEFAULT"])
        self.comment_prompt = PromptLibrary.get_prompt(IssueType["COMMENT"])
        self.ai_helper_prompt = PromptLibrary.get_prompt(IssueType["AIHELPER"])

    def run(self) -> None:
        import argcomplete

        prog_name: str = os.environ.get("CLI_NAME", os.path.basename(sys.argv[0]))
        parser: ArgumentParser = ArgumentParser(
            description="JIRA Issue Tool", prog=prog_name
        )
        subparsers: _SubParsersAction = parser.add_subparsers(
            dest="command", required=True
        )

        self._register_subcommands(subparsers)
        argcomplete.autocomplete(parser)
        args: Namespace = parser.parse_args()
        self._dispatch_command(args)

    def _register_subcommands(self, subparsers: _SubParsersAction) -> None:
        def add(name, help_text, aliases=None):
            return subparsers.add_parser(name, help=help_text, aliases=aliases or [])

        # --- 🧠 AI Helper ---
        ai_helper = add("ai-helper", "AI Helper")
        ai_helper.add_argument("prompt", help="A string describing a series of actions")
        ai_helper.add_argument("--voice", action="store_true")

        # --- 📌 Issue Creation & Editing ---
        create = add("create-issue", "Create a new issue")
        create.add_argument("type", help="bug, story, epic, task, spike")
        create.add_argument("summary", help="title of the issue")
        create.add_argument("--edit", action="store_true")
        create.add_argument("--dry-run", action="store_true")
        create.add_argument(
            "--lint",
            action="store_true",
            help="Run interactive linting on the description after AI cleanup",
        )

        edit = add("edit-issue", "Edit an issue's description")
        edit.add_argument("issue_key", help="The Jira issue id/key")
        edit.add_argument("--no-ai", action="store_true")
        edit.add_argument(
            "--lint",
            action="store_true",
            help="Run interactive linting on the description after AI cleanup",
        )

        # --- 🧾 Issue Metadata ---
        set_priority = add("set-priority", "Set issue priority")
        set_priority.add_argument("issue_key", help="The Jira issue id/key")
        set_priority.add_argument("priority", help="normal, major, critical")

        set_story_epic = add("set-story-epic", "Set stories epic")
        set_story_epic.add_argument("issue_key", help="The Jira issue id/key")
        set_story_epic.add_argument("epic_key", help="The Jira epic key")

        set_status = add("set-status", "Set issue status")
        set_status.add_argument("issue_key", help="The Jira issue id/key")
        set_status.add_argument("status", help="Closed, In Progress, Refinement, New")

        set_acceptance_criteria = add(
            "set-acceptance-criteria", "Set issue acceptance criteria"
        )
        set_acceptance_criteria.add_argument("issue_key", help="The Jira issue id/key")
        set_acceptance_criteria.add_argument(
            "acceptance_criteria", help="What needs to be done to accept it as complete"
        )

        change_type = add("change", "Change issue type")
        change_type.add_argument("issue_key", help="The Jira issue id/key")
        change_type.add_argument("new_type", help="bug, story, epic, task, spike")

        migrate = add("migrate", "Migrate issue to a new type")
        migrate.add_argument("issue_key", help="The Jira issue id/key")
        migrate.add_argument("new_type", help="bug, story, epic, task, spike")

        # --- 👤 Assignment & Workflow ---
        assign = add("assign", "Assign a user to an issue")
        assign.add_argument("issue_key", help="The Jira issue id/key")
        assign.add_argument("assignee", help="The person to assign it to")

        unassign = add("unassign", "Unassign a user from an issue")
        unassign.add_argument("issue_key", help="The Jira issue id/key")

        block = add("block", "Mark an issue as blocked")
        block.add_argument("issue_key", help="The Jira issue id/key")
        block.add_argument("reason", help="Reason the issue is blocked")

        unblock = add("unblock", "Mark an issue as unblocked")
        unblock.add_argument("issue_key", help="The Jira issue id/key")

        # --- 🧠 Estimation ---
        vote = add("vote-story-points", "Vote on story points")
        vote.add_argument("issue_key", help="The Jira issue id/key")
        vote.add_argument("points", help="Story point estimate (integer)")

        set_points = add("set-story-points", "Set story points directly")
        set_points.add_argument("issue_key", help="The Jira issue id/key")
        set_points.add_argument("points", help="Story point estimate (integer)")

        # --- 📅 Sprints ---
        add_sprint = add("add-sprint", "Add issue to sprint by name")
        add_sprint.add_argument("issue_key", help="The Jira issue id/key")
        add_sprint.add_argument("sprint_name", help="The name of the sprint")

        remove_sprint = add("remove-sprint", "Remove issue from its sprint")
        remove_sprint.add_argument("issue_key", help="The Jira issue id/key")

        # --- 💬 Comments ---
        comment = add("add-comment", "Add a comment to an issue")
        comment.add_argument("issue_key", help="The Jira issue id/key")
        comment.add_argument(
            "--text", help="Comment text (optional, otherwise opens $EDITOR)"
        )

        # --- 🔍 Issue Lookup ---
        search = add("search", "Search issues via JQL")
        search.add_argument("jql", help="JIRA Query Language expression")

        list_issues = add("list-issues", "List assigned issues")
        list_issues.add_argument("--project")
        list_issues.add_argument("--component")
        list_issues.add_argument("--assignee", help="Filter by JIRA issues by user")
        list_issues.add_argument(
            "--blocked", action="store_true", help="Show only blocked issues"
        )
        list_issues.add_argument(
            "--unblocked", action="store_true", help="Show only unblocked issues"
        )
        list_issues.add_argument("--status", help="Filter by JIRA status")
        list_issues.add_argument("--summary", help="Filter by summary text")
        list_issues.add_argument(
            "--show-reason",
            action="store_true",
            help="Show blocked reason field in listing",
        )
        list_issues.add_argument("--reporter", help="Filter by JIRA issues by user")

        lint = add("lint", "Lint an issue for quality")
        lint.add_argument("issue_key", help="The Jira issue id/key")

        lint_all = add("lint-all", "Lint all issues assigned to you")
        lint_all.add_argument("--project", help="Project key override")
        lint_all.add_argument("--component", help="Component filter")
        lint_all.add_argument("--assignee", help="Assignee filter")
        lint_all.add_argument("--reporter", help="Reporter filter")

        # --- 🔍 Viewers ---
        open_issue = add("open-issue", "Open issue in the browser")
        open_issue.add_argument("issue_key", help="The Jira issue id/key")

        view_issue = add("view-issue", "View issue in the console")
        view_issue.add_argument("issue_key", help="The Jira issue id/key")

        view_user = add("view-user", "Get and display a user")
        view_user.add_argument("account_id", help="Jira account ID")

        search_users = add("search-users", "Search for users by term")
        search_users.add_argument("query", help="Search term")

        blocked = add("blocked", "List blocked issues")
        blocked.add_argument("--user", help="Filter by assignee (username)")
        blocked.add_argument("--project", help="Optional project key")
        blocked.add_argument("--component", help="Optional component")

        # talk to jira
        talk = add("talk", "Talk to jira")
        talk.add_argument("--voice", action="store_true")

        # --- 📊 Reporting ---
        add("quarterly-connection", "Perform a quarterly connection report")

    def _dispatch_command(self, args: Namespace) -> None:
        try:
            getattr(self, args.command.replace("-", "_"))(args)
        except AttributeError as e:
            msg: str = f"❌ Command failed: {e}"
            print(msg)
            raise DispatcherError(msg)

    def ai_helper(self, args: Namespace) -> None:
        return cli_ai_helper(self, self.ai_provider, self.ai_helper_prompt, args)

    def open_issue(self, args: Namespace) -> None:
        return cli_open_issue(args)

    def view_issue(self, args: Namespace) -> None:
        return cli_view_issue(self.jira, args)

    def add_comment(self, args: Namespace) -> None:
        return cli_add_comment(self.jira, self.ai_provider, self.default_prompt, args)

    def create_issue(self, args: Namespace) -> None:
        return cli_create_issue(
            self.jira, self.ai_provider, self.default_prompt, self.template_dir, args
        )

    def list_issues(self, args: Namespace) -> None:
        return cli_list_issues(self.jira, args)

    def change_type(self, args: Namespace) -> None:
        return cli_change_type(self.jira, args)

    def migrate(self, args: Namespace) -> None:
        return cli_migrate(self.jira, args)

    def edit_issue(self, args: Namespace) -> None:
        return cli_edit_issue(
            self.jira, self.ai_provider, self.default_prompt, _try_cleanup, args
        )

    def _try_cleanup(self, prompt: str, text: str) -> str:
        return _try_cleanup(self.ai_provider, prompt, text)

    def unassign(self, args: Namespace) -> None:
        return cli_unassign(self.jira, args)

    def assign(self, args: Namespace) -> None:
        return cli_assign(self.jira, args)

    def set_priority(self, args: Namespace) -> None:
        return cli_set_priority(self.jira, args)

    def set_story_epic(self, args: Namespace) -> None:
        return cli_set_story_epic(self.jira, args)

    def remove_sprint(self, args: Namespace) -> None:
        return cli_remove_sprint(self.jira, args)

    def add_sprint(self, args: Namespace) -> None:
        return cli_add_sprint(self.jira, args)

    def set_status(self, args: Namespace) -> None:
        return cli_set_status(self.jira, args)

    def set_acceptance_criteria(self, args: Namespace) -> None:
        return cli_set_acceptance_criteria(self.jira, args)

    def vote_story_points(self, args: Namespace) -> None:
        return cli_vote_story_points(self.jira, args)

    def set_story_points(self, args: Namespace) -> None:
        return cli_set_story_points(self.jira, args)

    def block(self, args: Namespace) -> None:
        return cli_block(self.jira, args)

    def unblock(self, args: Namespace) -> None:
        return cli_unblock(self.jira, args)

    def validate_issue(self, fields: dict[str, str]) -> None:
        return cli_validate_issue(fields, self.ai_provider)

    def lint(self, args: Namespace) -> None:
        return cli_lint(self.jira, self.ai_provider, args)

    def lint_all(self, args: Namespace) -> None:
        return cli_lint_all(self.jira, self.ai_provider, args)

    def blocked(self, args: Namespace) -> None:
        return cli_blocked(self.jira, args)

    def search(self, args: Namespace) -> None:
        return cli_search(self.jira, args)

    def quarterly_connection(self, args: Namespace) -> None:
        return cli_quarterly_connection(self.jira, self.ai_provider)

    def search_users(self, args: Namespace) -> None:
        return cli_search_users(self.jira, args)

    def talk(self, args: Namespace) -> None:
        return cli_talk(self, args)

    def view_user(self, args: Namespace) -> None:
        return cli_view_user(self.jira, args)


if __name__ == "__main__":  # pragma: no cover
    JiraCLI().run()  # pragma: no cover



from unittest.mock import MagicMock

from core.env_fetcher import EnvFetcher


def test_set_sprint(client):
    client._request = MagicMock(return_value={})

    client.set_sprint("AAP-test_set_sprint", 42)

    client._request.assert_called_once_with(
        "PUT",
        "/rest/api/2/issue/AAP-test_set_sprint",
        json={"fields": {EnvFetcher.get("JIRA_SPRINT_FIELD"): ["42"]}},
    )


def test_remove_from_sprint(client):
    client._request = MagicMock(return_value={})

    client.remove_from_sprint("AAP-test_remove_from_sprint")

    client._request.assert_called_once_with(
        "POST",
        "/rest/agile/1.0/backlog/issue",
        json={"issues": ["AAP-test_remove_from_sprint"]},  # Matching the actual call
    )



import pytest
from templates.template_loader import TemplateLoader


def test_template_loader_parses_fields(tmp_path):
    # Create a simple template file
    template_content = (
        "FIELD|Title\n"
        "FIELD|Body\n"
        "TEMPLATE|Description\n"
        "Title: {{Title}}\n"
        "Body: {{Body}}"
    )
    tmpl_file = tmp_path / "story.tmpl"
    tmpl_file.write_text(template_content)

    loader = TemplateLoader(tmp_path, "story")
    fields = loader.get_fields()

    assert fields == ["Title", "Body"]


def test_template_loader_renders_description(tmp_path):
    template_content = (
        "FIELD|Topic\n" "TEMPLATE|Description\n" "You selected: {{Topic}}"
    )
    tmpl_file = tmp_path / "task.tmpl"
    tmpl_file.write_text(template_content)

    loader = TemplateLoader(tmp_path, "task")
    output = loader.render_description({"Topic": "Automation"})

    assert "You selected: Automation" in output


def test_template_loader_raises_file_not_found(tmp_path):
    # Use a temporary directory with no templates inside
    fake_template_dir = tmp_path
    issue_type = "nonexistent"

    with pytest.raises(FileNotFoundError) as excinfo:
        TemplateLoader(fake_template_dir, issue_type)

    assert f"{issue_type}.tmpl" in str(excinfo.value)


def test_get_template_returns_joined_string(tmp_path):
    template_file = tmp_path / "sample.tmpl"
    template_content = "FIELD|description\nTEMPLATE|\nline1\nline2\nline3"
    template_file.write_text(template_content)

    loader = TemplateLoader(template_dir=tmp_path, issue_type="sample")

    assert loader.template_lines == ["line1", "line2", "line3"]
    assert loader.get_template() == "line1\nline2\nline3"



from unittest.mock import MagicMock

import pytest
from exceptions.exceptions import ChangeIssueTypeError


def test_change_issue_type_fails(client, capsys):
    # Mock the _request method to raise an exception
    client._request = MagicMock(side_effect=ChangeIssueTypeError("failure"))

    with pytest.raises(ChangeIssueTypeError):
        # Attempt to change the issue type
        client.change_issue_type("AAP-test_change_issue_type_fails", "task")

    # Capture the output
    out = capsys.readouterr().out

    # Assert that the correct output was printed
    assert "❌ Failed to change issue type:" in out



import pytest


def test_add_to_sprint_board_id_check(client):
    # Mock the board_id attribute as None
    client.board_id = None

    # Check if the exception is raised when board_id is not set
    with pytest.raises(Exception, match="JIRA_BOARD_ID not set in environment"):
        client.add_to_sprint_by_name(
            "AAP-test_add_to_sprint_board_id_check", "Sprint Alpha"
        )



from unittest.mock import MagicMock

from core.env_fetcher import EnvFetcher


def mock_client_request(client, mock_return_value):
    # Mock get_current_user
    client.get_current_user = MagicMock(return_value="user123")

    # Mock the _request method to simulate an API response
    def mock_request(method, path, **kwargs):
        if method == "GET" and "search" in path:
            return mock_return_value

    client._request = MagicMock(side_effect=mock_request)


def test_list_issues(client):
    mock_client_request(client, {"issues": [{"key": "AAP-test_list_issues"}]})

    issues = client.list_issues(project="AAP", component="platform", assignee="user123")

    # Assert that the issues returned are a list and contain the correct key
    assert isinstance(issues, list)
    assert issues[0]["key"] == "AAP-test_list_issues"


def test_list_issues_reporter(client):
    mock_client_request(client, {"issues": [{"key": "AAP-test_list_issues_reporter"}]})

    issues = client.list_issues(project="AAP", component="platform", reporter="user123")

    # Assert that the issues returned are a list and contain the correct key
    assert isinstance(issues, list)
    assert issues[0]["key"] == "AAP-test_list_issues_reporter"


def test_list_issues_with_status(client):
    mock_client_request(
        client, {"issues": [{"key": "AAP-test_list_issues_with_status"}]}
    )

    issues = client.list_issues(
        project="AAP", component="platform", assignee="user123", status="In Progress"
    )

    # Assert that the issues returned are a list and contain the correct key
    assert isinstance(issues, list)
    assert issues[0]["key"] == "AAP-test_list_issues_with_status"


def test_list_issues_with_summary(client):
    mock_client_request(
        client, {"issues": [{"key": "AAP-test_list_issues_with_summary"}]}
    )

    issues = client.list_issues(
        project="AAP", component="platform", assignee="user123", summary="Onboarding"
    )

    # Assert that the issues returned are a list and contain the correct key
    assert isinstance(issues, list)
    assert issues[0]["key"] == "AAP-test_list_issues_with_summary"


def test_list_issues_with_blocked(client):
    mock_client_request(
        client, {"issues": [{"key": "AAP-test_list_issues_with_blocked"}]}
    )

    issues = client.list_issues(
        project="AAP", component="platform", assignee="user123", blocked=True
    )

    # Assert that the issues returned are a list and contain the correct key
    assert isinstance(issues, list)
    assert issues[0]["key"] == "AAP-test_list_issues_with_blocked"

    mock_client_request(
        client, {"issues": [{"key": "AAP-test_list_issues_with_blocked"}]}
    )


def test_list_issues_with_unblocked(client):
    mock_client_request(
        client, {"issues": [{"key": "AAP-test_list_issues_with_unblocked"}]}
    )

    issues = client.list_issues(
        project="AAP", component="platform", assignee="user123", unblocked=True
    )

    assert isinstance(issues, list)
    assert issues[0]["key"] == "AAP-test_list_issues_with_unblocked"

    mock_client_request(
        client, {"issues": [{"key": "AAP-test_list_issues_with_unblocked"}]}
    )


def test_list_issues_with_none_sprints(client):
    def mock_request(method, path, **kwargs):
        if method == "GET" and "search" in path:
            # Return an issue with 'JIRA_SPRINT_FIELD' set to None or missing
            return {
                "issues": [
                    {
                        "key": "AAP-test_list_issues_with_none_sprints",
                        "fields": {
                            "summary": "Run IQE tests in promotion pipelines",
                            "status": {"name": "In Progress"},
                            "assignee": {"displayName": "David O Neill"},
                            "priority": {"name": "Normal"},
                            EnvFetcher.get("JIRA_STORY_POINTS_FIELD"): 5,
                            EnvFetcher.get(
                                "JIRA_SPRINT_FIELD"
                            ): None,  # No sprints data
                        },
                    }
                ]
            }

    client._request = MagicMock(side_effect=mock_request)

    issues = client.list_issues(project="AAP", component="platform", assignee="user123")

    # Assert that the issues returned are a list and contain the correct key
    assert isinstance(issues, list)
    assert issues[0]["key"] == "AAP-test_list_issues_with_none_sprints"

    # Ensure that 'sprint' field is set to 'No active sprint' when sprints is None
    assert issues[0]["sprint"] == "No active sprint"


def test_list_issues_with_sprint_regex_matching(client):
    def mock_request(method, path, **kwargs):
        if method == "GET" and "search" in path:
            # Return an issue with JIRA_SPRINT_FIELD containing a sprint string
            return {
                "issues": [
                    {
                        "key": "AAP-test_list_issues_with_sprint_regex_matching",
                        "fields": {
                            "summary": "Run IQE tests in promotion pipelines",
                            "status": {"name": "In Progress"},
                            "assignee": {"displayName": "David O Neill"},
                            "priority": {"name": "Normal"},
                            EnvFetcher.get("JIRA_STORY_POINTS_FIELD"): 5,
                            EnvFetcher.get("JIRA_SPRINT_FIELD"): [
                                """com.atlassian.greenhopper.service.sprint.Sprint@5063ab17[id=70766,rapidViewId=18242,
                                state=ACTIVE,name=SaaS Sprint 2025-13,startDate=2025-03-27T12:01:00.000Z,"
                                endDate=2025-04-03T12:01:00.000Z]"""
                            ],  # Sprint data with ACTIVE state
                        },
                    }
                ]
            }

    client._request = MagicMock(side_effect=mock_request)

    issues = client.list_issues(project="AAP", component="platform", assignee="user123")

    # Assert that the issues returned are a list and contain the correct key
    # /* jscpd:ignore-start */
    assert isinstance(issues, list)
    assert issues[0]["key"] == "AAP-test_list_issues_with_sprint_regex_matching"
    # /* jscpd:ignore-end */
    # Ensure that the sprint is correctly extracted and assigned when sprint state is ACTIVE
    assert issues[0]["sprint"] == "SaaS Sprint 2025-13"  # Check the sprint name



from providers import get_ai_provider


def test_get_ai_provider_noop():
    provider = get_ai_provider("noop")
    assert provider.improve_text("prompt", "text") == "text"


def test_get_ai_provider_fallback():
    provider = get_ai_provider("xyz")
    assert provider.improve_text("prompt", "example") == "example"
